function [Ht_Forec,estPar,fVal] = dcc_garch(Data,p,q,r,m,alpha,beta,garchtype)

[T,k]  = size(Data);

HtMat  = [];

Specs  = garchset('Distribution', 'Gaussian', 'Display', 'off', 'VarianceModel', garchtype, 'P', p, 'Q', q,...
    'R', r, 'M', m);
errMat = [];
for i=1:k
    [Param, ~, ~, err, ht, ~] = garchfit(Specs,Data(:,i));
    [HForec,~]                = garchpred(Param,Data(:,i),1);
    if isnan(HForec)
        htMat(:,i) = ht(end);
    else
        htMat(:,i) = HForec;
    end
    htMat  = [ht;HForec];
    HtMat  = [HtMat,htMat];
    errMat = [errMat, err];
end

%for filtered ARMA residuals
errStart = errMat(1:1,:);
errMat   = errMat(2:end,:);

%for original returns
retStart = Data(1:1,:);
retMat   = Data(2:end,:);

%constructing the array of diagonal variance matrices
Dmat = zeros(k,k,T+1);
for i = 1:T+1   
    Dmat(:,:,i) = diag(HtMat(i,:)); 
end

%maximizing the loglikelihood function
parInit       = [alpha,beta];
options       = optimoptions('fmincon','Algorithm','active-set');
%options = optimset('MaxFunEvals',1000,'MaxIter',1000);
A             = [1,1;-1,0;0,-1];
b             = [0.9999,0,0]';

[estPar,fVal] = fmincon(@(corrPar)secdstepLL(errMat,errStart,Dmat,corrPar,k,T),parInit,A,b,[],[],[],[],[],options);

fVal = -fVal;

%obtaining the one-step-ahead forecast 
[~,Qhat,Q,QSt] = secdstepLL(errMat,errStart,Dmat,estPar,k,T);

Q_Forec        = (1 - estPar(1) - estPar(2)).*Qhat + estPar(1).*QSt{end}*((Dmat(:,:,end)^(-1))*errMat(end,:)')*((Dmat(:,:,end)^(-1))...
                   *errMat(end,:)')'*QSt{end} + estPar(2).*(Q{end});
    
QStInv         = diag((1./sqrt(diag(Q_Forec))));
R_Forec        = QStInv*Q_Forec*QStInv;


Ht_Forec       = (Dmat(:,:,end))*R_Forec*(Dmat(:,:,end));
end

%auxiliary function to estimate DCC log-likelihood
function [LL,Qhat,Q,QSt] = secdstepLL(errMat,errStart,Dmat,corrPar,k,T)

Qhat = zeros(k);

for i = 1:T-1
    qhat = ((Dmat(:,:,i)^(-1))*errMat(i,:)')*((Dmat(:,:,i)^(-1))*errMat(i,:)')';
    Qhat = Qhat + qhat;
end
Qhat   = (1/T).*Qhat;
Q{1}   = ((Dmat(:,:,1)^(-1))*errStart')*((Dmat(:,:,1)^(-1))*errStart')';
QSt{1} = diag((sqrt(diag(Q{1}))));

Q{2}   = (1 - corrPar(1) - corrPar(2)).*Qhat + corrPar(1).*QSt{1}*Q{1}*QSt{1} + corrPar(2).*Q{1};
QSt{2} = diag((sqrt(diag(Q{2}))));

for i = 3:size(errMat,1)
    
    Q{i}      = (1 - corrPar(1) - corrPar(2)).*Qhat + corrPar(1).*QSt{i-1}*((Dmat(:,:,i-1)^(-1))*errMat(i-1,:)')*((Dmat(:,:,i-1)^(-1))*errMat(i-1,:)')'*...
                 QSt{i-1} + corrPar(2).*cell2mat(Q(i-1));
    
    QSt{i}    = diag((sqrt(diag(Q{i}))));
    QStInv{i} = diag((1./sqrt(diag(Q{i}))));
    R{i}      = QStInv{i}*Q{i}*QStInv{i};
    
    ll(i)     = (2*log(det(R{i})) + ((Dmat(:,:,i-1)^(-1))*errMat(i,:)')'*((R{i})^(-1))*((Dmat(:,:,i-1)^(-1))*errMat(i,:)'));

end

LL = 0.5*sum(ll);

end