# close windows, clear variables
rm(list = ls(all=TRUE))
#specify the path to the desired folder, please set your own directory, e.g (C:/Users/Desktop/...)
setwd("~/Desktop/R")

#please download the following package
library(moments)
library(linprog)
library(quantreg)
library(fGarch)
library(nloptr)
library(pracma) 

#uploading hedge funds and S&P 500 returns data
dat                      =    read.csv("retdata.csv",header=F)
Date                     =    dat[,1]
Date                     =    as.numeric(Date-719529)
Date                     =    as.Date(Date,origin = '1970-01-01')

#S&P data matrix YRET
YRET                     =    dat[,ncol(dat)]
YRET                     =    as.data.frame(YRET)

#hedge funds data matrix XRET
XRET                     =    dat[,2:(ncol(dat)-1)]

IND                      =    as.matrix(YRET)
FUNDS                    =    as.matrix(XRET)

# set tau th quantile and set lambda
tau                      =    c(0.05,0.15,0.25,0.35,0.5)                                      #set of quantile indices
lambda                   =    seq(0.01,3,length=50)                                           #grid of lambdas            												
wwidth                   =    80
wshift                   =    0


#standard normal quantile for Cornish-Fisher expansion
cl                       =    0.01
z                        =    qnorm(cl,0,1)

#setting target return quantile level
TargRet                  =    0.7


#pre-allocating for caculation of capital 
cap                      =    list()
capp                     =    numeric()
cappp                    =    numeric() 
cap[[wwidth]]            =    1
capp[wwidth]             =    1
CAP                      =    numeric()

#pre-allocatiing for the arrays of CR-VaR results
BETA                     =    numeric()
VaRQ                     =    numeric()
II                       =    numeric()
WTS                      =    list()
COVM                     =    numeric()
INDEX                    =    list()
VaRQ                     =    numeric()

#cumuative returen calculation loop of stratergy 1
for (I in wwidth:length(FUNDS[,1])){
  if (IND[I] > quantile (IND[(1+wshift):I,], 0.5)){
    cap[[I]]               =    sum(as.matrix(as.data.frame(cap[I])))
    MeanRet                =    mean(IND[1:I,])
    sigma                  =    sd(IND[1:I,])
    skewn                  =    skewness (IND[1:I,])
    kurt                   =    kurtosis (IND[1:I,])
    VaR                    =    sum(as.matrix(as.data.frame(cap[I])))*(- (z + (1/6)*(z^2-1)*skewn+(1/24)*(z^3-3*z)*(kurt-3)-(1/36)*(2*z^3-5*z)*skewn^2)*sigma)
    VaR                    =    as.matrix(VaR)
    cap[[1+I]]             =    sum(as.matrix(as.data.frame(cap[I]))*(1+IND[I]))
    capp                   =    cap[[1+I]]
    cappp                  =    rbind(cappp,capp)
  } else {
    X                      =    XRET[(1+wshift):I,]
    Y                      =    YRET[(1+wshift):I,]
    X                      =    as.matrix(X)
    Y                      =    as.matrix(Y)

    n                      =    nrow(X) 
    k                      =    ncol(X)
    
    #lost function for Quantile
    Rhotau                 =    function(res,tau)
    {
      rhotau               =    sum(res*(tau - (res < 0)))              
    }
    
    #Setting up the ALQR optimization procedure                  
    #Step1: Lasso Quantile regression with non-positive constraint on beta.
    
    coeff                          =    numeric()                                                       #preallocating for the coefficients
    SIC                            =    numeric()
    numlam                         =    numeric()
    
    for (j in 1:length(lambda))
    {
      beta                         =    numeric()
      sic                          =    numeric()
      for (i in 1:length(tau))
      {                                                   
        bb                         =    rq.fit.lasso(X, Y, tau = tau[i], lambda = lambda[j], beta = .9995, eps = 1e-06)           #Solve Lasso Quantile Regression
        bbb                        =    bb$coefficients
        bbbb                       =    round(bbb,4)
        bbeta                      =    bbbb
        df                         =    length(bbeta[bbeta != 0])                                       #degree of freedom
        rhotau                     =    Rhotau(Y-X%*%bbeta,tau[i])                                      #calculating the rhotau
        ssic                       =    n*log(n^(-1)*rhotau) + log(n)*df                                #calculating the Bic statistic
        beta                       =    rbind(beta,bbeta)                                               #store the coefficients
        sic                        =    rbind(sic,ssic)                                                 #store the Bic statistics      
      }
      coeff                        =    rbind(coeff,beta)                                             
      SIC                          =    cbind(SIC,sic)                                                  #store the bic statistic according to different lambda
    }
    
    #choosing the coefficients at each quantile which correspond to the minimum bic statistic. 
    sicind05                       =    seq(1,length(tau)*length(lambda)-4, by=5)
    sicind15                       =    seq(2,length(tau)*length(lambda)-3, by=5)
    sicind25                       =    seq(3,length(tau)*length(lambda)-2, by=5)
    sicind35                       =    seq(4,length(tau)*length(lambda)-1, by=5)
    sicind50                       =    seq(5,length(tau)*length(lambda)-0, by=5)
    
    beta05                         =    coeff[sicind05,]
    beta15                         =    coeff[sicind15,]
    beta25                         =    coeff[sicind25,]
    beta35                         =    coeff[sicind35,]
    beta50                         =    coeff[sicind50,]
    
    sic05                          =    which.min(SIC[1,])
    sic15                          =    which.min(SIC[2,])
    sic25                          =    which.min(SIC[3,])
    sic35                          =    which.min(SIC[4,])
    sic50                          =    which.min(SIC[5,])
    sicall                         =    c(sic05,sic15,sic25,sic35,sic50)
    numlam                         =    rbind(numlam, sicall)
    
    Beta05                         =    beta05[sic05,]
    Beta15                         =    beta15[sic15,]
    Beta25                         =    beta25[sic25,]
    Beta35                         =    beta35[sic35,]
    Beta50                         =    beta50[sic50,]
    
    Betaall                        =    cbind(Beta05,Beta15,Beta25,Beta35,Beta50)
    #select the negative coefficients
    Betaall[which(Betaall>=-0.1)]  =    0.00
    BetaAll                        =    Betaall
    BETAFIN                        =    BetaAll
    
    #rule to invest into the funds which have significients at the quantile level 0.05 
    #if S&P 500 return is lower than its 5% empirical quantile
    
    if(IND[I] <= quantile (IND[(1+wshift):I,], 0.05)){
      ind                         =     (BETAFIN[,1] != 0)
      ind                         =     as.matrix (ind)
      RetMat                      =     subset(FUNDS[1:I,],select= ind)
      MeanRet                     =     as.matrix(apply(RetMat, 2, mean))
      P                           =     as.matrix(RetMat)
      
      if(ncol(P) == 0){
        cap[[I]]                  =    sum(as.matrix(as.data.frame(cap[I])))
        MeanRet                   =    mean(IND[1:I,])
        sigma                     =    sd(IND[1:I,])
        skewn                     =    skewness (IND[1:I,])
        kurt                      =    kurtosis (IND[1:I,])
        VaR                       =    sum(as.matrix(as.data.frame(cap[I])))*(- (z + (1/6)*(z^2-1)*skewn+(1/24)*(z^3-3*z)*(kurt-3)-(1/36)*(2*z^3-5*z)*skewn^2)*sigma)
        VaR                       =    as.matrix(VaR)
        cap[[1+I]]                =    sum(as.matrix(as.data.frame(cap[I]))*(1+IND[I]))
        capp                      =    cap[[1+I]]
        cappp                     =    rbind(cappp,capp)
      }else{ 
        #calculate covariance of the selected HF
        
        Sigma                       =     P
        for (i in 1:ncol(P)){
          k=garchFit(P[,i]~garch(1,1), data = P[,i])
          Sigma[,i]=k@residuals
        }
        M2                          =     cov(Sigma)
        CovMat                      =     M2
        
        M3                          =     numeric()
        for (ppi in 1:ncol(P))
        {
          SM3=array(rep(0,(ncol(P))^3),dim=c(ncol(P),ncol(P),ncol(P)))
          for (ppj in 1:ncol(P))
          {   
            for (ppk in 1:ncol(P))
            {
              u=0
              for (ppt in 1:nrow(P)) 
              {    
                u=u+(P[ppt,ppi])*(P[ppt,ppj])*(P[ppt,ppk]) 
              }    
              SM3[ppj,ppk,ppi]=u/(nrow(P))
            }                          
          }
          M3=cbind(M3,SM3[,,ppi])
        }
        
        
        M4                           =     numeric()
        
        for (ppi in 1:ncol(P))
        {
          for (ppj in 1:ncol(P))
          {
            SM4=array(rep(0,(ncol(P))^4),c(ncol(P),ncol(P),ncol(P),ncol(P)))
            for (ppk in 1:ncol(P))
            {   
              for (ppm in 1:ncol(P))
              {
                u=0
                for (ppt in 1:nrow(P)) 
                {    
                  u=u+(P[ppt,ppi])*(P[ppt,ppj])*(P[ppt,ppk])*(P[ppt,ppm]) 
                }    
                SM4[ppk,ppm,ppj,ppi]=u/(nrow(P))
              }                          
            }
            M4=cbind(M4,SM4[,,ppj,ppi])
          }
        }
        
        #setting up the Cornish-Fisher Value-at-Risk objective function
        # Sp          = (1/((t(w) %*% M2 %*% w)^(3/2)))*(t(w) %*% M3 %*% kron(w,w)-3*((t(w)%*% M2 %*% w)+(t(w) %*% MeanRet)^2)*(t(w) %*% MeanRet)+2*((t(w) %*% MeanRet)^3))
        # Kp          = (1/((t(w) %*% M2 %*% w)^2))*(t(w) %*% M4 %*% kron (kron(w,w))-4*(t(w) %*% M3 %*% kron(w,w))*(t(w) %*% MeanRet)+6*(t(w)%*% M2 %*% w) %*% (t(w) %*% MeanRet)^2+3*(t(w) %*% MeanRet))-3
        # q           = z+ (1/6) *(z^2-1)*( Sp ) + (1/24) * (z^3-3*z)*( Kp ) - (1/36) * (2*z^3-5*z)*( Sp )^2 
        
        
        
        eval_f             = function(w) {
          return(( - (z+(z^2-1)*(((1/((t(w)%*%M2%*%w)^(3/2)))*(t(w)%*%M3%*%kron(w,w)-3*(t(w)%*%M2%*%w + (t(w)%*%MeanRet)^2)*(t(w)%*%MeanRet)+2*((t(w)%*%MeanRet)^3)))/6) + (z^3-3*z)*((((1/((t(w)%*%M2%*%w)^2))*(t(w)%*%M4%*%kron(kron(w,w),w)-4*t(w)%*%M3%*%kron(w,w)*(t(w)%*%MeanRet)+6*(t(w)%*%M2%*%w + (t(w)%*%MeanRet)^2)*(t(w)%*%MeanRet)^2+3*(t(w)%*%MeanRet)^4))-3)/24) - (2*z^3-5*z)*(((((1/((t(w)%*%M2%*%w)^(3/2)))*(t(w)%*%M3%*%kron(w,w)-3*(t(w)%*%M2%*%w + (t(w)%*%MeanRet)^2)*(t(w)%*%MeanRet)+2*((t(w)%*%MeanRet)^3))))^2)/36))*(sqrt(t(w)%*%M2%*%w)))
          )
        }
        x0                 = matrix(rep(1,ncol(P)),nrow=ncol(P))/ncol(P)
        MeanRet            = as.matrix(apply(P,2,mean))
        
        eval_g_eq          = function(x){
          return(c((sum(x)-1),t(x)%*%MeanRet-0.7))
        }
        opts               = list("algorithm"="NLOPT_GN_ISRES",
                                  "xtol_rel"=1.0e-8,
                                  "maxeval"=10000)
        
        res                = nloptr( x0=x0, 
                                     eval_f=eval_f, 
                                     eval_g_eq=eval_g_eq,
                                     lb = rep(0,length(x0)), 
                                     ub = rep(1,length(x0)),
                                     opts=opts)
        print( res )
        num_digw           =    4
        wts= res$solution
        wts=as.matrix(wts)
        wts = round(wts*(10^num_digw))/(10^num_digw)
        cap[[I]]                =    sum(as.matrix(as.data.frame(cap[I])))
        cap[[1+I]]              =    sum(as.matrix(as.data.frame(cap[I]))*sum(wts*(1+RetMat[I,])))
        capp                    =    cap[[1+I]]
        cappp                   =    rbind(cappp, capp)
        wts=as.matrix(wts)
      }#else if(ncol(p)== 1) 
    }else if(IND[I] <= quantile (IND[(1+wshift):I,], 0.15)&& IND[I] > quantile (IND[(1+wshift):I,], 0.05)){
      #rule to invest into the funds which have significients at the quantile level 0.05 
      #if S&P 500 return is hight than its 5% empirical quantile 
      #and lower than its 15% empirical quantile
      ind                       =  (BETAFIN[,2] != 0)
      ind                       =  as.matrix (ind)
      RetMat                    =  subset(FUNDS[1:I,],select= ind)
      MeanRet                   =  as.matrix(apply(RetMat, 2, mean))
      P                         =  RetMat
      
      if(ncol(P) == 0){
        cap[[I]]                  =    sum(as.matrix(as.data.frame(cap[I])))
        MeanRet                   =    mean(IND[1:I,])
        sigma                     =    sd(IND[1:I,])
        skewn                     =    skewness (IND[1:I,])
        kurt                      =    kurtosis (IND[1:I,])
        VaR                       =    sum(as.matrix(as.data.frame(cap[I])))*(- (z + (1/6)*(z^2-1)*skewn+(1/24)*(z^3-3*z)*(kurt-3)-(1/36)*(2*z^3-5*z)*skewn^2)*sigma)
        VaR                       =    as.matrix(VaR)
        cap[[1+I]]                =    sum(as.matrix(as.data.frame(cap[I]))*(1+IND[I]))
        capp                      =    cap[[1+I]]
        cappp                     =    rbind(cappp,capp)
      }else{
        #calculate covariance of the selected HF
        
        Sigma           = P
        for (i in 1:ncol(P)){
          k=garchFit(P[,i]~garch(1,1), data = P[,i])
          Sigma[,i]=k@residuals
        }
        M2              = cov(Sigma)
        CovMat          = M2
        
        M3              = numeric()
        for (ppi in 1:ncol(P))
        {
          SM3=array(rep(0,(ncol(P))^3),dim=c(ncol(P),ncol(P),ncol(P)))
          for (ppj in 1:ncol(P))
          {   
            for (ppk in 1:ncol(P))
            {
              u=0
              for (ppt in 1:nrow(P)) 
              {    
                u=u+(P[ppt,ppi])*(P[ppt,ppj])*(P[ppt,ppk]) 
              }    
              SM3[ppj,ppk,ppi]=u/(nrow(P))
            }                          
          }
          M3=cbind(M3,SM3[,,ppi])
        }
        
        
        M4              = numeric()
        
        for (ppi in 1:ncol(P))
        {
          for (ppj in 1:ncol(P))
          {
            SM4=array(rep(0,(ncol(P))^4),c(ncol(P),ncol(P),ncol(P),ncol(P)))
            for (ppk in 1:ncol(P))
            {   
              for (ppm in 1:ncol(P))
              {
                u=0
                for (ppt in 1:nrow(P)) 
                {    
                  u=u+(P[ppt,ppi])*(P[ppt,ppj])*(P[ppt,ppk])*(P[ppt,ppm]) 
                }    
                SM4[ppk,ppm,ppj,ppi]=u/(nrow(P))
              }                          
            }
            M4=cbind(M4,SM4[,,ppj,ppi])
          }
        }
        
        #setting up the Cornish-Fisher Value-at-Risk objective function
        # Sp          = (1/((t(w) %*% M2 %*% w)^(3/2)))*(t(w) %*% M3 %*% kron(w,w)-3*((t(w)%*% M2 %*% w)+(t(w) %*% MeanRet)^2)*(t(w) %*% MeanRet)+2*((t(w) %*% MeanRet)^3))
        # Kp          = (1/((t(w) %*% M2 %*% w)^2))*(t(w) %*% M4 %*% kron (kron(w,w))-4*(t(w) %*% M3 %*% kron(w,w))*(t(w) %*% MeanRet)+6*(t(w)%*% M2 %*% w) %*% (t(w) %*% MeanRet)^2+3*(t(w) %*% MeanRet))-3
        # q           = z+ (1/6) *(z^2-1)*( Sp ) + (1/24) * (z^3-3*z)*( Kp ) - (1/36) * (2*z^3-5*z)*( Sp )^2 
        
        
        
        eval_f             = function(w) {
          return(( - (z+(z^2-1)*(((1/((t(w)%*%M2%*%w)^(3/2)))*(t(w)%*%M3%*%kron(w,w)-3*(t(w)%*%M2%*%w + (t(w)%*%MeanRet)^2)*(t(w)%*%MeanRet)+2*((t(w)%*%MeanRet)^3)))/6) + (z^3-3*z)*((((1/((t(w)%*%M2%*%w)^2))*(t(w)%*%M4%*%kron(kron(w,w),w)-4*t(w)%*%M3%*%kron(w,w)*(t(w)%*%MeanRet)+6*(t(w)%*%M2%*%w + (t(w)%*%MeanRet)^2)*(t(w)%*%MeanRet)^2+3*(t(w)%*%MeanRet)^4))-3)/24) - (2*z^3-5*z)*(((((1/((t(w)%*%M2%*%w)^(3/2)))*(t(w)%*%M3%*%kron(w,w)-3*(t(w)%*%M2%*%w + (t(w)%*%MeanRet)^2)*(t(w)%*%MeanRet)+2*((t(w)%*%MeanRet)^3))))^2)/36))*(sqrt(t(w)%*%M2%*%w)))
          )
        }
        x0                 = matrix(rep(1,ncol(P)),nrow=ncol(P))/ncol(P)
        MeanRet            = as.matrix(apply(P,2,mean))
        
        eval_g_eq          = function(x){
          return(c((sum(x)-1),t(x)%*%MeanRet-0.7))
        }
        opts               = list("algorithm"="NLOPT_GN_ISRES",
                                  "xtol_rel"=1.0e-8,
                                  "maxeval"=10000)
        
        res                = nloptr( x0=x0, 
                                     eval_f=eval_f, 
                                     eval_g_eq=eval_g_eq,
                                     lb = rep(0,length(x0)), 
                                     ub = rep(1,length(x0)),
                                     opts=opts)
        print( res )
        num_digw                 =    4
        wts= res$solution
        wts=as.matrix(wts)
        wts = round(wts*(10^num_digw))/(10^num_digw)
        cap[[I]]               =    sum(as.matrix(as.data.frame(cap[I])))
        cap[[1+I]]             =    sum(as.matrix(as.data.frame(cap[I]))*sum(wts*(1+RetMat[I,])))
        capp                   =    cap[[1+I]]
        cappp                  =    rbind(cappp, capp)
        wts=as.matrix(wts)
      }#else if(ncol(p)== 1)     
    }else if(IND[I] <= quantile (IND[(1+wshift):I,], 0.25)&& IND[I] > quantile (IND[(1+wshift):I,], 0.15)){
      #rule to invest into the funds which have significients at the quantile level 0.05 
      #if S&P 500 return is hight than its 15% empirical quantile 
      #and lower than its 25% empirical quantile
      ind                       =  (BETAFIN[,3] != 0)
      ind                       =  as.matrix (ind)
      RetMat                    =  subset(FUNDS[1:I,],select= ind)
      MeanRet                   =  as.matrix(apply(RetMat, 2, mean))
      P                         =  RetMat
      
      if(ncol(P) == 0){
        cap[[I]]                  =    sum(as.matrix(as.data.frame(cap[I])))
        MeanRet                   =    mean(IND[1:I,])
        sigma                     =    sd(IND[1:I,])
        skewn                     =    skewness (IND[1:I,])
        kurt                      =    kurtosis (IND[1:I,])
        VaR                       =    sum(as.matrix(as.data.frame(cap[I])))*(- (z + (1/6)*(z^2-1)*skewn+(1/24)*(z^3-3*z)*(kurt-3)-(1/36)*(2*z^3-5*z)*skewn^2)*sigma)
        VaR                       =    as.matrix(VaR)
        cap[[1+I]]                =    sum(as.matrix(as.data.frame(cap[I]))*(1+IND[I]))
        capp                      =    cap[[1+I]]
        cappp                     =    rbind(cappp,capp)
      }else{
        #calculate covariance of the selected HF
        
        Sigma           = P
        for (i in 1:ncol(P)){
          k=garchFit(P[,i]~garch(1,1), data = P[,i])
          Sigma[,i]=k@residuals
        }
        M2              = cov(Sigma)
        CovMat          = M2
        
        M3              = numeric()
        for (ppi in 1:ncol(P))
        {
          SM3=array(rep(0,(ncol(P))^3),dim=c(ncol(P),ncol(P),ncol(P)))
          for (ppj in 1:ncol(P))
          {   
            for (ppk in 1:ncol(P))
            {
              u=0
              for (ppt in 1:nrow(P)) 
              {    
                u=u+(P[ppt,ppi])*(P[ppt,ppj])*(P[ppt,ppk]) 
              }    
              SM3[ppj,ppk,ppi]=u/(nrow(P))
            }                          
          }
          M3=cbind(M3,SM3[,,ppi])
        }
        
        
        M4              = numeric()
        
        for (ppi in 1:ncol(P))
        {
          for (ppj in 1:ncol(P))
          {
            SM4=array(rep(0,(ncol(P))^4),c(ncol(P),ncol(P),ncol(P),ncol(P)))
            for (ppk in 1:ncol(P))
            {   
              for (ppm in 1:ncol(P))
              {
                u=0
                for (ppt in 1:nrow(P)) 
                {    
                  u=u+(P[ppt,ppi])*(P[ppt,ppj])*(P[ppt,ppk])*(P[ppt,ppm]) 
                }    
                SM4[ppk,ppm,ppj,ppi]=u/(nrow(P))
              }                          
            }
            M4=cbind(M4,SM4[,,ppj,ppi])
          }
        }
        
        #setting up the Cornish-Fisher Value-at-Risk objective function
        # Sp          = (1/((t(w) %*% M2 %*% w)^(3/2)))*(t(w) %*% M3 %*% kron(w,w)-3*((t(w)%*% M2 %*% w)+(t(w) %*% MeanRet)^2)*(t(w) %*% MeanRet)+2*((t(w) %*% MeanRet)^3))
        # Kp          = (1/((t(w) %*% M2 %*% w)^2))*(t(w) %*% M4 %*% kron (kron(w,w))-4*(t(w) %*% M3 %*% kron(w,w))*(t(w) %*% MeanRet)+6*(t(w)%*% M2 %*% w) %*% (t(w) %*% MeanRet)^2+3*(t(w) %*% MeanRet))-3
        # q           = z+ (1/6) *(z^2-1)*( Sp ) + (1/24) * (z^3-3*z)*( Kp ) - (1/36) * (2*z^3-5*z)*( Sp )^2 
         
        eval_f             = function(w) {
          return(( - (z+(z^2-1)*(((1/((t(w)%*%M2%*%w)^(3/2)))*(t(w)%*%M3%*%kron(w,w)-3*(t(w)%*%M2%*%w + (t(w)%*%MeanRet)^2)*(t(w)%*%MeanRet)+2*((t(w)%*%MeanRet)^3)))/6) + (z^3-3*z)*((((1/((t(w)%*%M2%*%w)^2))*(t(w)%*%M4%*%kron(kron(w,w),w)-4*t(w)%*%M3%*%kron(w,w)*(t(w)%*%MeanRet)+6*(t(w)%*%M2%*%w + (t(w)%*%MeanRet)^2)*(t(w)%*%MeanRet)^2+3*(t(w)%*%MeanRet)^4))-3)/24) - (2*z^3-5*z)*(((((1/((t(w)%*%M2%*%w)^(3/2)))*(t(w)%*%M3%*%kron(w,w)-3*(t(w)%*%M2%*%w + (t(w)%*%MeanRet)^2)*(t(w)%*%MeanRet)+2*((t(w)%*%MeanRet)^3))))^2)/36))*(sqrt(t(w)%*%M2%*%w)))
          )
        }
        x0                 = matrix(rep(1,ncol(P)),nrow=ncol(P))/ncol(P)
        MeanRet            = as.matrix(apply(P,2,mean))
        
        eval_g_eq          = function(x){
          return(c((sum(x)-1),t(x)%*%MeanRet-0.7))
        }
        opts               = list("algorithm"="NLOPT_GN_ISRES",
                                  "xtol_rel"=1.0e-8,
                                  "maxeval"=10000)
        
        res                = nloptr( x0=x0, 
                                     eval_f=eval_f, 
                                     eval_g_eq=eval_g_eq,
                                     lb = rep(0,length(x0)), 
                                     ub = rep(1,length(x0)),
                                     opts=opts)
        print( res )
        num_digw                 =    4
        wts= res$solution
        wts=as.matrix(wts)
        wts = round(wts*(10^num_digw))/(10^num_digw)
        cap[[I]]               =    sum(as.matrix(as.data.frame(cap[I])))
        cap[[1+I]]             =    sum(as.matrix(as.data.frame(cap[I]))*sum(wts*(1+RetMat[I,])))
        capp                   =    cap[[1+I]]
        cappp                  =    rbind(cappp, capp)
        wts=as.matrix(wts)
      }# if (ncol(P) == 0) else
    }else if(IND[I] <= quantile (IND[(1+wshift):I,], 0.35)&& IND[I] > quantile (IND[(1+wshift):I,], 0.25)){
      #rule to invest into the funds which have significients at the quantile level 0.05 
      #if S&P 500 return is hight than its 25% empirical quantile 
      #and lower than its 35% empirical quantile
      ind                       =  (BETAFIN[,4] != 0)
      ind                       =  as.matrix (ind)
      RetMat                    =  subset(FUNDS[1:I,],select= ind)
      MeanRet                   =  as.matrix(apply(RetMat, 2, mean))
      P                         =  RetMat
      
      if(ncol(P) == 0){
        cap[[I]]                  =    sum(as.matrix(as.data.frame(cap[I])))
        MeanRet                   =    mean(IND[1:I,])
        sigma                     =    sd(IND[1:I,])
        skewn                     =    skewness (IND[1:I,])
        kurt                      =    kurtosis (IND[1:I,])
        VaR                       =    sum(as.matrix(as.data.frame(cap[I])))*(- (z + (1/6)*(z^2-1)*skewn+(1/24)*(z^3-3*z)*(kurt-3)-(1/36)*(2*z^3-5*z)*skewn^2)*sigma)
        VaR                       =    as.matrix(VaR)
        cap[[1+I]]                =    sum(as.matrix(as.data.frame(cap[I]))*(1+IND[I]))
        capp                      =    cap[[1+I]]
        cappp                     =    rbind(cappp,capp)
      }else{
        #calculate covariance of the selected HF
        
        Sigma           = P
        for (i in 1:ncol(P)){
          k=garchFit(P[,i]~garch(1,1), data = P[,i])
          Sigma[,i]=k@residuals
        }
        M2              = cov(Sigma)
        CovMat          = M2
        
        M3              = numeric()
        for (ppi in 1:ncol(P))
        {
          SM3=array(rep(0,(ncol(P))^3),dim=c(ncol(P),ncol(P),ncol(P)))
          for (ppj in 1:ncol(P))
          {   
            for (ppk in 1:ncol(P))
            {
              u=0
              for (ppt in 1:nrow(P)) 
              {    
                u=u+(P[ppt,ppi])*(P[ppt,ppj])*(P[ppt,ppk]) 
              }    
              SM3[ppj,ppk,ppi]=u/(nrow(P))
            }                          
          }
          M3=cbind(M3,SM3[,,ppi])
        }
        
        
        M4              = numeric()
        
        for (ppi in 1:ncol(P))
        {
          for (ppj in 1:ncol(P))
          {
            SM4=array(rep(0,(ncol(P))^4),c(ncol(P),ncol(P),ncol(P),ncol(P)))
            for (ppk in 1:ncol(P))
            {   
              for (ppm in 1:ncol(P))
              {
                u=0
                for (ppt in 1:nrow(P)) 
                {    
                  u=u+(P[ppt,ppi])*(P[ppt,ppj])*(P[ppt,ppk])*(P[ppt,ppm]) 
                }    
                SM4[ppk,ppm,ppj,ppi]=u/(nrow(P))
              }                          
            }
            M4=cbind(M4,SM4[,,ppj,ppi])
          }
        }
        
        #setting up the Cornish-Fisher Value-at-Risk objective function
        # Sp          = (1/((t(w) %*% M2 %*% w)^(3/2)))*(t(w) %*% M3 %*% kron(w,w)-3*((t(w)%*% M2 %*% w)+(t(w) %*% MeanRet)^2)*(t(w) %*% MeanRet)+2*((t(w) %*% MeanRet)^3))
        # Kp          = (1/((t(w) %*% M2 %*% w)^2))*(t(w) %*% M4 %*% kron (kron(w,w))-4*(t(w) %*% M3 %*% kron(w,w))*(t(w) %*% MeanRet)+6*(t(w)%*% M2 %*% w) %*% (t(w) %*% MeanRet)^2+3*(t(w) %*% MeanRet))-3
        # q           = z+ (1/6) *(z^2-1)*( Sp ) + (1/24) * (z^3-3*z)*( Kp ) - (1/36) * (2*z^3-5*z)*( Sp )^2 
        
       
        eval_f             = function(w) {
          return(( - (z+(z^2-1)*(((1/((t(w)%*%M2%*%w)^(3/2)))*(t(w)%*%M3%*%kron(w,w)-3*(t(w)%*%M2%*%w + (t(w)%*%MeanRet)^2)*(t(w)%*%MeanRet)+2*((t(w)%*%MeanRet)^3)))/6) + (z^3-3*z)*((((1/((t(w)%*%M2%*%w)^2))*(t(w)%*%M4%*%kron(kron(w,w),w)-4*t(w)%*%M3%*%kron(w,w)*(t(w)%*%MeanRet)+6*(t(w)%*%M2%*%w + (t(w)%*%MeanRet)^2)*(t(w)%*%MeanRet)^2+3*(t(w)%*%MeanRet)^4))-3)/24) - (2*z^3-5*z)*(((((1/((t(w)%*%M2%*%w)^(3/2)))*(t(w)%*%M3%*%kron(w,w)-3*(t(w)%*%M2%*%w + (t(w)%*%MeanRet)^2)*(t(w)%*%MeanRet)+2*((t(w)%*%MeanRet)^3))))^2)/36))*(sqrt(t(w)%*%M2%*%w)))
          )
        }
        x0                 = matrix(rep(1,ncol(P)),nrow=ncol(P))/ncol(P)
        MeanRet            = as.matrix(apply(P,2,mean))
        
        eval_g_eq          = function(x){
          return(c((sum(x)-1),t(x)%*%MeanRet-0.7))
        }
        opts               = list("algorithm"="NLOPT_GN_ISRES",
                                  "xtol_rel"=1.0e-8,
                                  "maxeval"=10000)
        
        res                = nloptr( x0=x0, 
                                     eval_f=eval_f, 
                                     eval_g_eq=eval_g_eq,
                                     lb = rep(0,length(x0)), 
                                     ub = rep(1,length(x0)),
                                     opts=opts)
        print( res )
        num_digw                 =    4
        wts= res$solution
        wts=as.matrix(wts)
        wts = round(wts*(10^num_digw))/(10^num_digw)
        cap[[I]]               =    sum(as.matrix(as.data.frame(cap[I])))
        cap[[1+I]]             =    sum(as.matrix(as.data.frame(cap[I]))*sum(wts*(1+RetMat[I,])))
        capp                   =    cap[[1+I]]
        cappp                  =    rbind(cappp, capp)
        wts=as.matrix(wts)
      }#if(ncol(P) == 0),else
    }else{
      #rule to invest into the funds which have significients at the quantile level 0.05 
      #if S&P 500 return is hight than its 35% empirical quantile 
      #and lower than its 50% empirical quantile
      ind                       =  (BETAFIN[,5] != 0)
      ind                       =  as.matrix (ind)
      RetMat                    =  subset(FUNDS[1:I,],select= ind)
      MeanRet                   =  as.matrix(apply(RetMat, 2, mean))
      P                         =  RetMat
      
      if(ncol(P) == 0){
        cap[[I]]                  =    sum(as.matrix(as.data.frame(cap[I])))
        MeanRet                   =    mean(IND[1:I,])
        sigma                     =    sd(IND[1:I,])
        skewn                     =    skewness (IND[1:I,])
        kurt                      =    kurtosis (IND[1:I,])
        VaR                       =    sum(as.matrix(as.data.frame(cap[I])))*(- (z + (1/6)*(z^2-1)*skewn+(1/24)*(z^3-3*z)*(kurt-3)-(1/36)*(2*z^3-5*z)*skewn^2)*sigma)
        VaR                       =    as.matrix(VaR)
        cap[[1+I]]                =    sum(as.matrix(as.data.frame(cap[I]))*(1+IND[I]))
        capp                      =    cap[[1+I]]
        cappp                     =    rbind(cappp,capp)
      }else{
        #calculate covariance of the selected HF
       
        Sigma           = P
        for (i in 1:ncol(P)){
          k=garchFit(P[,i]~garch(1,1), data = P[,i])
          Sigma[,i]=k@residuals
        }
        M2              = cov(Sigma)
        CovMat          = M2
        
        M3              = numeric()
        for (ppi in 1:ncol(P))
        {
          SM3=array(rep(0,(ncol(P))^3),dim=c(ncol(P),ncol(P),ncol(P)))
          for (ppj in 1:ncol(P))
          {   
            for (ppk in 1:ncol(P))
            {
              u=0
              for (ppt in 1:nrow(P)) 
              {    
                u=u+(P[ppt,ppi])*(P[ppt,ppj])*(P[ppt,ppk]) 
              }    
              SM3[ppj,ppk,ppi]=u/(nrow(P))
            }                          
          }
          M3=cbind(M3,SM3[,,ppi])
        }
        
        
        M4              = numeric()
        
        for (ppi in 1:ncol(P))
        {
          for (ppj in 1:ncol(P))
          {
            SM4=array(rep(0,(ncol(P))^4),c(ncol(P),ncol(P),ncol(P),ncol(P)))
            for (ppk in 1:ncol(P))
            {   
              for (ppm in 1:ncol(P))
              {
                u=0
                for (ppt in 1:nrow(P)) 
                {    
                  u=u+(P[ppt,ppi])*(P[ppt,ppj])*(P[ppt,ppk])*(P[ppt,ppm]) 
                }    
                SM4[ppk,ppm,ppj,ppi]=u/(nrow(P))
              }                          
            }
            M4=cbind(M4,SM4[,,ppj,ppi])
          }
        }
        
        #setting up the Cornish-Fisher Value-at-Risk objective function
        # Sp          = (1/((t(w) %*% M2 %*% w)^(3/2)))*(t(w) %*% M3 %*% kron(w,w)-3*((t(w)%*% M2 %*% w)+(t(w) %*% MeanRet)^2)*(t(w) %*% MeanRet)+2*((t(w) %*% MeanRet)^3))
        # Kp          = (1/((t(w) %*% M2 %*% w)^2))*(t(w) %*% M4 %*% kron (kron(w,w))-4*(t(w) %*% M3 %*% kron(w,w))*(t(w) %*% MeanRet)+6*(t(w)%*% M2 %*% w) %*% (t(w) %*% MeanRet)^2+3*(t(w) %*% MeanRet))-3
        # q           = z+ (1/6) *(z^2-1)*( Sp ) + (1/24) * (z^3-3*z)*( Kp ) - (1/36) * (2*z^3-5*z)*( Sp )^2 
        
       
        eval_f             = function(w) {
          return(( - (z+(z^2-1)*(((1/((t(w)%*%M2%*%w)^(3/2)))*(t(w)%*%M3%*%kron(w,w)-3*(t(w)%*%M2%*%w + (t(w)%*%MeanRet)^2)*(t(w)%*%MeanRet)+2*((t(w)%*%MeanRet)^3)))/6) + (z^3-3*z)*((((1/((t(w)%*%M2%*%w)^2))*(t(w)%*%M4%*%kron(kron(w,w),w)-4*t(w)%*%M3%*%kron(w,w)*(t(w)%*%MeanRet)+6*(t(w)%*%M2%*%w + (t(w)%*%MeanRet)^2)*(t(w)%*%MeanRet)^2+3*(t(w)%*%MeanRet)^4))-3)/24) - (2*z^3-5*z)*(((((1/((t(w)%*%M2%*%w)^(3/2)))*(t(w)%*%M3%*%kron(w,w)-3*(t(w)%*%M2%*%w + (t(w)%*%MeanRet)^2)*(t(w)%*%MeanRet)+2*((t(w)%*%MeanRet)^3))))^2)/36))*(sqrt(t(w)%*%M2%*%w)))
          )
        }
        x0                 = matrix(rep(1,ncol(P)),nrow=ncol(P))/ncol(P)
        MeanRet            = as.matrix(apply(P,2,mean))
        
        eval_g_eq          = function(x){
          return(c((sum(x)-1),t(x)%*%MeanRet-0.7))
        }
        opts               = list("algorithm"="NLOPT_GN_ISRES",
                                  "xtol_rel"=1.0e-8,
                                  "maxeval"=10000)
        
        res                = nloptr( x0=x0, 
                                     eval_f=eval_f, 
                                     eval_g_eq=eval_g_eq,
                                     lb = rep(0,length(x0)), 
                                     ub = rep(1,length(x0)),
                                     opts=opts)
        print( res )
        num_digw               =    4
        wts= res$solution
        wts= as.matrix(wts)
        wts= round(wts*(10^num_digw))/(10^num_digw)
        cap[[I]]               =    sum(as.matrix(as.data.frame(cap[I])))
        cap[[1+I]]             =    sum(as.matrix(as.data.frame(cap[I]))*sum(wts*(1+RetMat[I,])))
        capp                   =    cap[[1+I]]
        cappp                  =    rbind(cappp, capp)
        wts                    =    as.matrix(wts)
      }#if(ncol(P) == 0)  
    }#0.35-0.5,else
    BETA                     =    rbind(BETA, BETAFIN)
    WTS                      =    list(WTS,wts)
    INDEX                    =    list(INDEX,ind)   
  } #if (IND[I] > quantile (IND[(1+wshift):I,], 0.5)), else
    VaRQ                     =    rbind(VaRQ, VaR)
  wshift                     =    wshift+1
}   #for (I in wwidth:length(FUNDS[,1]))
CAP                          =    rbind(CAP,cappp)


#Stragtegy "buy and hold"
INDCAP                       =    list()
INDCAPIT                     =    numeric()
INDCAPITST                   =    numeric() 
INDCAP[[wwidth]]             =    1
ishift                       =    0

for (i in wwidth:(length(IND[,1])-1)){
  INDCAP[[i+1]]              =    INDCAP[[i]]*(1+IND[i,1])
  indcaplast                 =    INDCAP[[i+1]]
  INDCAPIT                   =    rbind(INDCAPIT,indcaplast)
  ishift                     =    ishift+1
}
INDCAPITST                   =    rbind(1,INDCAPIT)


Time                         =    as.Date(Date[(wwidth-1):(length(FUNDS[,1])-1)],)

plot(Time, INDCAPITST, type = "l",main="Comparison of Strategies", ylab = "Cumulative Return",col="blue",ylim=c(-1,6))
lines(Time, CAP, col="red")