clc
clear
load('hf_bloom_ret.mat') %pre-loading data 
load('hf_bloom_tick.mat')

% load('hedfhd_short.mat') %pre-loading data 
% load('hedfhdnames_short.mat')

Date         = HFBloomRet(1:end,1);
Date         = [Date;datenum('31-Jan-2013')];
XRET         = HFBloomRet(1:end,2:end-1); %creating covariate and response matrices
YRET         = HFBloomRet(1:end,end);
% IND        = YRET./100;
% FUNDS      = XRET./100;

IND          = YRET;
FUNDS        = XRET;

tau          = [0.05, 0.15, 0.25, 0.35, 0.50]; %set of quantile indices
lambda       = linspace(0.01,3,50); %grid of lambdas
wwidth       = 80; %moving window width
wshift       = 0;

pp           = 1;
qq           = 1;
rr           = 1;
mm           = 1;
alpha        = 0.2; %starting DCC parameters
betta        = 0.7;
garchtype    = 'GARCH';

cl = 0.01; 
z=norminv(cl,0,1); %standard normal quantile for Cornish-Fisher expansion

num_digw     = 4;

TargRet      = 0.7; %target return quantile level

% pre-allocating for the arrays to be created
clear cap capp cappp
cap{wwidth}  = 1; 
capp(wwidth) = 1;
cappp        = [];
BETA         = [];
numlam       = [];
ll           = [];
VaRQ         = [];
WTS          = {};
COVM         = {};
INDEX        = {};
EstPar       = [];
Specs        = garchset('Distribution', 'Gaussian', 'Display', 'off', 'VarianceModel', 'GARCH', 'P', 1, 'Q', 1,...
    'R', 1, 'M', 1);

%STRATEGY 1 CUMULATIVE RETURN CALCULATION LOOP
for l = wwidth:size(FUNDS,1)   
    
    if IND(l) > quantile(IND(1+wshift:l,:),0.50) %investing into S&P500 if its return is higher than median
        cap{l}    = sum(cell2mat(cap(l)));
        MeanRet   = mean(IND(1:l,:)); %calculating the mean return vector
        sigma     = std(IND(1:l,:)); %calculating the S&P500 position volatility
        skewn     = skewness(IND(1:l,:)); %calculating the S&P500 position skewness
        kurt      = kurtosis(IND(1:l,:)); %calculating the S&P500 position kurtosis
        %calculating the S&P500 position risk
        %MeanRet
        VaR       = sum(cell2mat(cap(l)))*( - (z + (1/6)*(z^2-1)*skewn+(1/24)*(z^3-3*z)*(kurt-3)-(1/36)*(2*z^3-5*z)*skewn^2)*sigma);
        %portfolio value appreciation
        cap{l+1}  = sum(cell2mat(cap(l)).*(1 + IND(l)));
        capp      = cap{l+1};
        cappp     = [cappp,capp];
    else
        %setting up the ALQR optimization procedure
            X     = XRET(1+wshift:l,:);
            Y     = YRET(1+wshift:l,:);
            [n,k] = size(X);
    
            coeff = [];
            SIC   = []; %preallocating for the BIC statistic
            for j = 1:length(lambda)
    
                beta  = [];
                sic   = [];
                for i = 1:length(tau)
                   
                    c          = [tau(i).*ones(1,n),(1-tau(i)).*ones(1,n),lambda(j).*ones(1,k),zeros(1,k)];
                    A          = [eye(n),-eye(n),zeros(n,k),X];
                    A          = sparse(A);
                    B          =  [-eye(k,n),zeros(k,n),zeros(k,k),zeros(k,k); 
                                    zeros(k,n),-eye(k,n),zeros(k,k),zeros(k,k);
                                    zeros(k,n),zeros(k,n),-eye(k,k),eye(k,k);
                                    zeros(k,n),zeros(k,n),-eye(k,k),-eye(k,k)
                                    zeros(k,n),zeros(k,n),zeros(k,k),eye(k,k)];
                    B          = sparse(B);
                    
                    q          = [zeros(1,k),zeros(1,k),zeros(1,k),zeros(1,k),zeros(1,k)]';
                    b          = Y;
                    lb         = [zeros(1,n),zeros(1,n),-Inf.*ones(1,k),-Inf.*ones(1,k)];
                    ub         = [Inf.*ones(1,n),Inf.*ones(1,n),Inf.*ones(1,k),zeros(1,k)];
                    options    = optimset('LargeScale','on','MaxIter',2000);
                    bb         = linprog(c,B,q,A,b,lb,ub,[],options);
                    bbeta      = bb(length(bb) - k + 1:end);
                    df         = length(bbeta(bbeta ~= 0)); %calculating the BIC statistic 
                    qrLoss     = sum(qrRho(Y - X*bbeta,tau(i)));
                    sicc       = n*log(n^(-1)*qrLoss) + log(n)*df;
                    beta       = [beta,bbeta];
                    sic        = [sic,sicc];
                end
                coeff = [coeff,beta];
                SIC   = [SIC;sic];
            end
            
            %choosing the coefficients at each quantile which correspond to
            %the minimum SIC statistic
            sicind05    = 1:5:length(tau)*length(lambda)-4;
            sicind15    = 2:5:length(tau)*length(lambda)-3;
            sicind25    = 3:5:length(tau)*length(lambda)-2;
            sicind35    = 4:5:length(tau)*length(lambda)-1;
            sicind50    = 5:5:length(tau)*length(lambda)-0;

            beta05      = coeff(:,sicind05);
            beta15      = coeff(:,sicind15);
            beta25      = coeff(:,sicind25);
            beta35      = coeff(:,sicind35);
            beta50      = coeff(:,sicind50);

            sic05       = find(SIC(:,1) == min(SIC(:,1)));
            sic15       = find(SIC(:,2) == min(SIC(:,2)));
            sic25       = find(SIC(:,3) == min(SIC(:,3)));
            sic35       = find(SIC(:,4) == min(SIC(:,4)));
            sic50       = find(SIC(:,5) == min(SIC(:,5)));
            sicall      = [sic05, sic15, sic25, sic35, sic50];
            numlam      = [numlam; sicall];

            Beta05      = beta05(:,sic05);
            Beta15      = beta15(:,sic15);
            Beta25      = beta25(:,sic25);
            Beta35      = beta35(:,sic35);
            Beta50      = beta50(:,sic50);

            BetaAll     = [Beta05,Beta15,Beta25,Beta35,Beta50];
            
            %creating the weighted covariate matrix for the Adaptive Lasso  
            clear XWW
            XWW = {};
            for i = 1:length(tau)
                XW = [];
                for j = 1:n
                    XWi = X(j,:).*max(abs(BetaAll(:,i)'),1/sqrt(n));
                    XW  = [XW;XWi];
                end
                XWW = [XWW,XW];
            end
            
            BetaLam = round(BetaAll.*(10^num_digw))./(10^num_digw);
            
            %rule-of-thumb lambda (see the text) 
            Lambda = 0.25*sqrt(sum(abs(BetaLam) > 0))*log(k)*(log(n))^(0.1/2);
            COEFF  = [];
            %estimating the coefficients again with the new weighted
            %covariate matrix
            for i       = 1:length(Lambda)
                c       = [tau(i).*ones(1,n),(1-tau(i)).*ones(1,n),Lambda(i).*ones(1,k),zeros(1,k)];
                A       = [eye(n),-eye(n),zeros(n,k),XWW{i}];
                A       = sparse(A);
                B       =  [-eye(k,n),zeros(k,n),zeros(k,k),zeros(k,k); 
                             zeros(k,n),-eye(k,n),zeros(k,k),zeros(k,k);
                             zeros(k,n),zeros(k,n),-eye(k,k),eye(k,k);
                             zeros(k,n),zeros(k,n),-eye(k,k),-eye(k,k)
                             zeros(k,n),zeros(k,n),zeros(k,k),eye(k,k)];
                B       = sparse(B);
                q       = [zeros(1,k),zeros(1,k),zeros(1,k),zeros(1,k),zeros(1,k)]';
                b       = Y;
                lb      = [zeros(1,n),zeros(1,n),-Inf.*ones(1,k),-Inf.*ones(1,k)];
                ub      = [Inf.*ones(1,n),Inf.*ones(1,n),Inf.*ones(1,k),zeros(1,k)];
                options = optimset('LargeScale','on','MaxIter',2000);
                bb      = linprog(c,B,q,A,b,lb,ub,[],options);
                COE     = bb(length(bb) - k + 1:end);
                COEFF   = [COEFF,COE];   
            end
            %calculating the final Adaptive Lasso coefficients
            Coeff       = COEFF.*max(abs(BetaLam),1/sqrt(n));
    
            BETAFIN     = round(Coeff.*(10^num_digw))./(10^num_digw);
            options=optimset('Algorithm','active-set');

            if IND(l) <= quantile(IND(1+wshift:l,:),0.05) %rule to invest into the funds which have 
                %significant coefficients at the quantile level 0.05 if S&P500 return is lower than its 5% empirical quantile 
                ind     = find(BETAFIN(:,1) ~= 0);
                RetMat  = FUNDS(1:l,ind);
                w0      = ones(1,size(RetMat,2))./size(RetMat,2);
                MeanRet = mean(RetMat)';
                P       = RetMat;
                %estimating the DCC model to estimate the time-changing
                %covariance structure
                if size(P,2) > 1
                    %[~, ~ ,Ht, ~, ~, ~] = dcc(P,[],1,0,1,1,0,1,2,'3-stage','none',[],[]);
                    [Ht,estPar,~] = dcc_garch(P,pp,qq,rr,mm,alpha,betta,garchtype);
                    M2            = Ht(:,:,end);
                elseif size(P,2) == 1                   
                    [Parameters, ~, ~, ~, ~, ~] = garchfit(Specs,P);
                    [HForec,~]                  = garchpred(Parameters,P,1);
                    M2                          = HForec^2;
                else
                    cap{l}   = sum(cell2mat(cap(l)));
                    MeanRet  = mean(IND(1:l,:)); %calculating the mean return vector
                    sigma    = std(IND(1:l,:)); %calculating the S&P500 position volatility
                    skewn    = skewness(IND(1:l,:)); %calculating the S&P500 position skewness
                    kurt     = kurtosis(IND(1:l,:)); %calculating the S&P500 position kurtosis
                    %calculating the S&P500 position risk
                    VaR      = sum(cell2mat(cap(l)))*( - (z + (1/6)*(z^2-1)*skewn+(1/24)*(z^3-3*z)*(kurt-3)-(1/36)*(2*z^3-5*z)*skewn^2)*sigma);
                    %portfolio value appreciation
                    cap{l+1} = sum(cell2mat(cap(l)).*(1 + IND(l)));
                    capp     = cap{l+1};
                    cappp    = [cappp,capp]  
                    VaRQ     = [VaRQ,VaR] 
                    wshift   = wshift + 1  
                    ll       = [ll,l]
                    continue                     
                end
                CovMat = M2;
                %estimating the co-skewness and co-kurtosis tensors 
                M3     = [];
                for i=1:size(P,2)
                    S=[];
                     for j=1:size(P,2)
                        for k=1:size(P,2)
                            u=0;
                            for t=1:size(P,1) 
                                u=u+((P(t,i))*(P(t,j)) ...
                                    *(P(t,k)));
                            end
                            S(j,k)=u/(size(P,1)); 
                        end
                     end
                    M3=[M3 S];
                end
                M4=[];
                for i=1:size(P,2)
                    for j=1:size(P,2)
                        S=[];
                        for k=1:size(P,2)
                            for m=1:size(P,2)
                                u=0;
                                for t=1:size(P,1)
                                    u=u+((P(t,i))*(P(t,j))* ...
                                        (P(t,k))*(P(t,m)));
                                end
                                S(k,m)=u/(size(P,1));
                            end
                        end
                        M4=[M4 S];
                    end
                end
                %setting up the Cornish-Fisher Value-at-Risk objective
                %function
                wub       = ones(length(w0),1);
                wlb       = zeros(length(w0),1);
                Aeq       = ones(1,length(w0));
                beq       = 1;
                AA        = -MeanRet';
                bb        = -quantile(MeanRet,TargRet);
                cap{l}    = sum(cell2mat(cap(l)));
                [wts,VaR] = fmincon(@(w)cap{l}*( - (z+(z^2-1)*(((1/((w*M2*w')^(3/2)))*(w*M3*kron(w',w')-3*(w*M2*w' + (w*MeanRet)^2)*...
                           (w*MeanRet)+2*((w*MeanRet)^3)))/6) + (z^3-3*z)*((((1/((w*M2*w')^2))*(w*M4*kron(kron(w',w'),w')-4*w*M3*kron(w',w')*...
                           (w*MeanRet)+6*(w*M2*w' + (w*MeanRet)^2)*(w*MeanRet)^2+3*(w*MeanRet)^4))-3)/24) - (2*z^3-5*z)*(((((1/((w*M2*w')^(3/2)))*...
                           (w*M3*kron(w',w')-3*(w*M2*w' + (w*MeanRet)^2)*(w*MeanRet)+2*((w*MeanRet)^3))))^2)/36))*(sqrt(w*M2*w'))),w0,AA,bb,Aeq,...
                            beq,wlb,wub,[],options);
                wts       = round(wts.*(10^num_digw))./(10^num_digw);
                %portfolio value appreciation
                cap{l+1}  = sum(cell2mat(cap(l)).*wts'.*(1 + FUNDS(l,ind)'));
                capp      = cap{l+1};    
                cappp     = [cappp,capp]
            elseif IND(l) <= quantile(IND(1+wshift:l,:),0.15)&& IND(l) > quantile(IND(1+wshift:l,:),0.05)%rule to invest into the funds which have 
                %significant coefficients at the quantile level 0.05 if S&P500 return is higher than its 5% empirical quantile and
                %lower than its 15% empirical quantile (the rest of calculations repeats itself in the if-loop)
                ind = find(BETAFIN(:,2) ~= 0);
                RetMat = FUNDS(1:l,ind);
                w0 = ones(1,size(RetMat,2))./size(RetMat,2);
                MeanRet = mean(RetMat)';
                P = RetMat;
                if size(P,2) > 1
                    %[~, ~ ,Ht, ~, ~, ~] = dcc(P,[],1,0,1,1,0,1,2,'3-stage','none',[],[]);
                    [Ht,estPar,~]               = dcc_garch(P,pp,qq,rr,mm,alpha,betta,garchtype);
                    M2 = Ht(:,:,end);
                elseif size(P,2) == 1                   
                    [Parameters, ~, ~, ~, ~, ~] = garchfit(Specs,P);
                    [HForec,~]                  = garchpred(Parameters,P,1);
                    M2                          = HForec^2;
                else
                    cap{l}   = sum(cell2mat(cap(l)));
                    MeanRet  = mean(IND(1:l,:)); %calculating the mean return vector
                    sigma    = std(IND(1:l,:)); %calculating the S&P500 position volatility
                    skewn    = skewness(IND(1:l,:)); %calculating the S&P500 position skewness
                    kurt     = kurtosis(IND(1:l,:)); %calculating the S&P500 position kurtosis
                    %calculating the S&P500 position risk
                    VaR      = sum(cell2mat(cap(l)))*( - (z + (1/6)*(z^2-1)*skewn+(1/24)*(z^3-3*z)*(kurt-3)-(1/36)*(2*z^3-5*z)*skewn^2)*sigma);
                    %portfolio value appreciation
                    cap{l+1} = sum(cell2mat(cap(l)).*(1 + IND(l)));
                    capp     = cap{l+1};
                    cappp    = [cappp,capp] 
                    VaRQ     = [VaRQ,VaR] 
                    wshift   = wshift + 1 
                    ll       = [ll,l]
                    continue                     
                end
                CovMat = M2;
                M3     =[];
                for i=1:size(P,2)
                    S=[];
                     for j=1:size(P,2)
                        for k=1:size(P,2)
                            u=0;
                            for t=1:size(P,1) 
                                u=u+((P(t,i))*(P(t,j)) ...
                                    *(P(t,k)));
                            end
                            S(j,k)=u/(size(P,1)); 
                        end
                     end
                    M3=[M3 S];
                end
                M4=[];
                for i=1:size(P,2)
                    for j=1:size(P,2)
                        S=[];
                        for k=1:size(P,2)
                            for m=1:size(P,2)
                                u=0;
                                for t=1:size(P,1)
                                    u=u+((P(t,i))*(P(t,j))* ...
                                        (P(t,k))*(P(t,m)));
                                end
                                S(k,m)=u/(size(P,1));
                            end
                        end
                        M4=[M4 S];
                    end
                end
                wub       = ones(length(w0),1);
                wlb       = zeros(length(w0),1);
                Aeq       = ones(1,length(w0));
                beq       = 1;
                AA        = -MeanRet';
                bb        = -quantile(MeanRet,TargRet);
                cap{l}    = sum(cell2mat(cap(l)));
                %w*MeanRet
                [wts,VaR] = fmincon(@(w)cap{l}*( - (z+(z^2-1)*(((1/((w*M2*w')^(3/2)))*(w*M3*kron(w',w')-3*(w*M2*w' + (w*MeanRet)^2)*...
                           (w*MeanRet)+2*((w*MeanRet)^3)))/6) + (z^3-3*z)*((((1/((w*M2*w')^2))*(w*M4*kron(kron(w',w'),w')-4*w*M3*kron(w',w')*...
                           (w*MeanRet)+6*(w*M2*w' + (w*MeanRet)^2)*(w*MeanRet)^2+3*(w*MeanRet)^4))-3)/24) - (2*z^3-5*z)*(((((1/((w*M2*w')^(3/2)))*...
                           (w*M3*kron(w',w')-3*(w*M2*w' + (w*MeanRet)^2)*(w*MeanRet)+2*((w*MeanRet)^3))))^2)/36))*(sqrt(w*M2*w'))),w0,AA,bb,Aeq,...
                            beq,wlb,wub,[],options);
                wts       = round(wts.*(10^num_digw))./(10^num_digw);
                cap{l+1}  = sum(cell2mat(cap(l)).*wts'.*(1 + FUNDS(l,ind)'));
                capp      = cap{l+1};    
                cappp     = [cappp,capp]
            elseif IND(l) <= quantile(IND(1+wshift:l,:),0.25)&& IND(l) > quantile(IND(1+wshift:l,:),0.15)%rule to invest into the funds which have 
                %significant coefficients at the quantile level 0.05 if S&P500 return is higher than its 15% empirical quantile and
                %lower than its 25% empirical quantile (the rest of calculations repeats itself in the if-loop)
                ind       = find(BETAFIN(:,3) ~= 0);
                RetMat    = FUNDS(1:l,ind);
                w0        = ones(1,size(RetMat,2))./size(RetMat,2);
                MeanRet   = mean(RetMat)';
                P         = RetMat;
                if size(P,2) > 1
                    %[~, ~ ,Ht, ~, ~, ~] = dcc(P,[],1,0,1,1,0,1,2,'3-stage','none',[],[]);
                    [Ht,estPar,~]               = dcc_garch(P,pp,qq,rr,mm,alpha,betta,garchtype);
                    M2                          = Ht(:,:,end);
                elseif size(P,2) == 1                   
                    [Parameters, ~, ~, ~, ~, ~] = garchfit(Specs,P);
                    [HForec,~]                  = garchpred(Parameters,P,1);
                    M2                          = HForec^2;
                else
                    cap{l}                      = sum(cell2mat(cap(l)));
                    MeanRet                     = mean(IND(1:l,:)); %calculating the mean return vector
                    sigma                       = std(IND(1:l,:)); %calculating the S&P500 position volatility
                    skewn                       = skewness(IND(1:l,:)); %calculating the S&P500 position skewness
                    kurt                        = kurtosis(IND(1:l,:)); %calculating the S&P500 position kurtosis
                    %calculating the S&P500 position risk
                    VaR                         = sum(cell2mat(cap(l)))*( - (z + (1/6)*(z^2-1)*skewn+(1/24)*(z^3-3*z)*(kurt-3)-(1/36)*(2*z^3-5*z)*skewn^2)*sigma);
                    %portfolio value appreciation
                    cap{l+1}                    = sum(cell2mat(cap(l)).*(1 + IND(l)));
                    capp                        = cap{l+1};
                    cappp                       = [cappp,capp]
                    VaRQ                        = [VaRQ,VaR] 
                    wshift                      = wshift + 1 
                    ll                          = [ll,l]
                    continue                     
                end
                CovMat = M2;
                M3     =[];
                for i=1:size(P,2)
                    S=[];
                     for j=1:size(P,2)
                        for k=1:size(P,2)
                            u=0;
                            for t=1:size(P,1) 
                                u=u+((P(t,i))*(P(t,j)) ...
                                    *(P(t,k)));
                            end
                            S(j,k)=u/(size(P,1)); 
                        end
                     end
                    M3=[M3 S];
                end
                M4=[];
                for i=1:size(P,2)
                    for j=1:size(P,2)
                        S=[];
                        for k=1:size(P,2)
                            for m=1:size(P,2)
                                u=0;
                                for t=1:size(P,1)
                                    u=u+((P(t,i))*(P(t,j))* ...
                                        (P(t,k))*(P(t,m)));
                                end
                                S(k,m)=u/(size(P,1));
                            end
                        end
                        M4=[M4 S];
                    end
                end
                wub       = ones(length(w0),1);
                wlb       = zeros(length(w0),1);
                Aeq       = ones(1,length(w0));
                beq       = 1;
                AA        = -MeanRet';
                bb        = -quantile(MeanRet,TargRet);
                cap{l}    = sum(cell2mat(cap(l)));
                %w*MeanRet
                [wts,VaR] = fmincon(@(w)cap{l}*( - (z+(z^2-1)*(((1/((w*M2*w')^(3/2)))*(w*M3*kron(w',w')-3*(w*M2*w' + (w*MeanRet)^2)*...
                           (w*MeanRet)+2*((w*MeanRet)^3)))/6) + (z^3-3*z)*((((1/((w*M2*w')^2))*(w*M4*kron(kron(w',w'),w')-4*w*M3*kron(w',w')*...
                           (w*MeanRet)+6*(w*M2*w' + (w*MeanRet)^2)*(w*MeanRet)^2+3*(w*MeanRet)^4))-3)/24) - (2*z^3-5*z)*(((((1/((w*M2*w')^(3/2)))*...
                           (w*M3*kron(w',w')-3*(w*M2*w' + (w*MeanRet)^2)*(w*MeanRet)+2*((w*MeanRet)^3))))^2)/36))*(sqrt(w*M2*w'))),w0,AA,bb,Aeq,...
                            beq,wlb,wub,[],options);
                wts       = round(wts.*(10^num_digw))./(10^num_digw);
                cap{l+1}  = sum(cell2mat(cap(l)).*wts'.*(1 + FUNDS(l,ind)'));
                capp      = cap{l+1};    
                cappp     = [cappp,capp]
            elseif IND(l) <= quantile(IND(1+wshift:l,:),0.35)&& IND(l) > quantile(IND(1+wshift:l,:),0.25)%rule to invest into the funds which have 
                %significant coefficients at the quantile level 0.05 if S&P500 return is higher than its 25% empirical quantile and
                %lower than its 35% empirical quantile (the rest of calculations repeats itself in the if-loop)
                ind       = find(BETAFIN(:,4) ~= 0);
                RetMat    = FUNDS(1:l,ind);
                w0        = ones(1,size(RetMat,2))./size(RetMat,2);
                MeanRet   = mean(RetMat)';
                P         = RetMat;
                if size(P,2) > 1
                    %[~, ~ ,Ht, ~, ~, ~] = dcc(P,[],1,0,1,1,0,1,2,'3-stage','none',[],[]);
                    [Ht,estPar,~]               = dcc_garch(P,pp,qq,rr,mm,alpha,betta,garchtype);
                    M2                          = Ht(:,:,end);
                elseif size(P,2) == 1                   
                    [Parameters, ~, ~, ~, ~, ~] = garchfit(Specs,P);
                    [HForec,~]                  = garchpred(Parameters,P,1);
                    M2                          = HForec^2;
                else
                    cap{l}                      = sum(cell2mat(cap(l)));
                    MeanRet                     = mean(IND(1:l,:)); %calculating the mean return vector
                    sigma                       = std(IND(1:l,:)); %calculating the S&P500 position volatility
                    skewn                       = skewness(IND(1:l,:)); %calculating the S&P500 position skewness
                    kurt                        = kurtosis(IND(1:l,:)); %calculating the S&P500 position kurtosis
                    %calculating the S&P500 position risk
                    VaR                         = sum(cell2mat(cap(l)))*( - (z + (1/6)*(z^2-1)*skewn+(1/24)*(z^3-3*z)*(kurt-3)-(1/36)*(2*z^3-5*z)*skewn^2)*sigma);
                    %portfolio value appreciation
                    cap{l+1}                    = sum(cell2mat(cap(l)).*(1 + IND(l)));
                    capp                        = cap{l+1};
                    cappp                       = [cappp,capp]  
                    VaRQ                        = [VaRQ,VaR] 
                    wshift                      = wshift + 1  
                    ll                          = [ll,l]
                    continue                     
                end
                CovMat = M2;
                M3     = [];
                for i=1:size(P,2)
                    S=[];
                     for j=1:size(P,2)
                        for k=1:size(P,2)
                            u=0;
                            for t=1:size(P,1) 
                                u=u+((P(t,i))*(P(t,j)) ...
                                    *(P(t,k)));
                            end
                            S(j,k)=u/(size(P,1)); 
                        end
                     end
                    M3=[M3 S];
                end
                M4=[];
                for i=1:size(P,2)
                    for j=1:size(P,2)
                        S=[];
                        for k=1:size(P,2)
                            for m=1:size(P,2)
                                u=0;
                                for t=1:size(P,1)
                                    u=u+((P(t,i))*(P(t,j))* ...
                                        (P(t,k))*(P(t,m)));
                                end
                                S(k,m)=u/(size(P,1));
                            end
                        end
                        M4=[M4 S];
                    end
                end
                wub       = ones(length(w0),1);
                wlb       = zeros(length(w0),1);
                Aeq       = ones(1,length(w0));
                beq       = 1;
                AA        = -MeanRet';
                bb        = -quantile(MeanRet,TargRet);
                cap{l}    = sum(cell2mat(cap(l)));
                %w*MeanRet
                [wts,VaR] = fmincon(@(w)cap{l}*( - (z+(z^2-1)*(((1/((w*M2*w')^(3/2)))*(w*M3*kron(w',w')-3*(w*M2*w' + (w*MeanRet)^2)*...
                    (w*MeanRet)+2*((w*MeanRet)^3)))/6) + (z^3-3*z)*((((1/((w*M2*w')^2))*(w*M4*kron(kron(w',w'),w')-4*w*M3*kron(w',w')*...
                    (w*MeanRet)+6*(w*M2*w' + (w*MeanRet)^2)*(w*MeanRet)^2+3*(w*MeanRet)^4))-3)/24) - (2*z^3-5*z)*(((((1/((w*M2*w')^(3/2)))*...
                    (w*M3*kron(w',w')-3*(w*M2*w' + (w*MeanRet)^2)*(w*MeanRet)+2*((w*MeanRet)^3))))^2)/36))*(sqrt(w*M2*w'))),w0,AA,bb,Aeq,...
                    beq,wlb,wub,[],options);
                wts       = round(wts.*(10^num_digw))./(10^num_digw);
                cap{l+1}  = sum(cell2mat(cap(l)).*wts'.*(1 + FUNDS(l,ind)'));
                capp      = cap{l+1};    
                cappp     = [cappp,capp]
            else %rule to invest into the funds which have 
                %significant coefficients at the quantile level 0.05 if S&P500 return is higher than its 35% empirical quantile
                %and lower than its median
                ind       = find(BETAFIN(:,5) ~= 0);
                RetMat    = FUNDS(1:l,ind);
                w0        = ones(1,size(RetMat,2))./size(RetMat,2);
                MeanRet   = mean(RetMat)';
                P         = RetMat;
                if size(P,2) > 1
                    %[~, ~ ,Ht, ~, ~, ~] = dcc(P,[],1,0,1,1,0,1,2,'3-stage','none',[],[]);
                    [Ht,estPar,~] = dcc_garch(P,pp,qq,rr,mm,alpha,betta,garchtype);
                    M2            = Ht(:,:,end);
                elseif size(P,2) == 1                   
                    [Parameters, ~, ~, ~, ~, ~] = garchfit(Specs,P);
                    [HForec,~]                  = garchpred(Parameters,P,1);
                    M2                          = HForec^2;
                else
                    cap{l}                      = sum(cell2mat(cap(l)));
                    MeanRet                     = mean(IND(1:l,:)); %calculating the mean return vector
                    sigma                       = std(IND(1:l,:)); %calculating the S&P500 position volatility
                    skewn                       = skewness(IND(1:l,:)); %calculating the S&P500 position skewness
                    kurt                        = kurtosis(IND(1:l,:)); %calculating the S&P500 position kurtosis
                    %calculating the S&P500 position risk
                    VaR                         = sum(cell2mat(cap(l)))*( - (z + (1/6)*(z^2-1)*skewn+(1/24)*(z^3-3*z)*(kurt-3)-(1/36)*(2*z^3-5*z)*skewn^2)*sigma);
                    %portfolio value appreciation
                    cap{l+1}                    = sum(cell2mat(cap(l)).*(1 + IND(l)));
                    capp                        = cap{l+1};
                    cappp                       = [cappp,capp]
                    VaRQ                        = [VaRQ,VaR] 
                    wshift                      = wshift + 1
                    ll                          = [ll,l]
                    continue                     
                end
                CovMat = M2;
                M3     = [];
                for i=1:size(P,2)
                    S=[];
                     for j=1:size(P,2)
                        for k=1:size(P,2)
                            u=0;
                            for t=1:size(P,1) 
                                u=u+((P(t,i))*(P(t,j)) ...
                                    *(P(t,k)));
                            end
                            S(j,k)=u/(size(P,1)); 
                        end
                     end
                    M3=[M3 S];
                end
                M4=[];
                for i=1:size(P,2)
                    for j=1:size(P,2)
                        S=[];
                        for k=1:size(P,2)
                            for m=1:size(P,2)
                                u=0;
                                for t=1:size(P,1)
                                    u=u+((P(t,i))*(P(t,j))* ...
                                        (P(t,k))*(P(t,m)));
                                end
                                S(k,m)=u/(size(P,1));
                            end
                        end
                        M4=[M4 S];
                    end
                end
                wub       = ones(length(w0),1);
                wlb       = zeros(length(w0),1);
                Aeq       = ones(1,length(w0));
                beq       = 1;
                AA        = -MeanRet';
                bb        = -quantile(MeanRet,TargRet);
                cap{l}    = sum(cell2mat(cap(l)));
                %w*MeanRet
                [wts,VaR] = fmincon(@(w)cap{l}*( - (z+(z^2-1)*(((1/((w*M2*w')^(3/2)))*(w*M3*kron(w',w')-3*(w*M2*w' + (w*MeanRet)^2)*...
                           (w*MeanRet)+2*((w*MeanRet)^3)))/6) + (z^3-3*z)*((((1/((w*M2*w')^2))*(w*M4*kron(kron(w',w'),w')-4*w*M3*kron(w',w')*...
                           (w*MeanRet)+6*(w*M2*w' + (w*MeanRet)^2)*(w*MeanRet)^2+3*(w*MeanRet)^4))-3)/24) - (2*z^3-5*z)*(((((1/((w*M2*w')^(3/2)))*...
                           (w*M3*kron(w',w')-3*(w*M2*w' + (w*MeanRet)^2)*(w*MeanRet)+2*((w*MeanRet)^3))))^2)/36))*(sqrt(w*M2*w'))),w0,AA,bb,Aeq,...
                            beq,wlb,wub,[],options);
                wts       = round(wts.*(10^num_digw))./(10^num_digw);
                cap{l+1}  = sum(cell2mat(cap(l)).*wts'.*(1 + FUNDS(l,ind)'));
                capp      = cap{l+1};    
                cappp     = [cappp,capp]
        
            end            
            BETA   = [BETA,BETAFIN];
            WTS    = [WTS;wts]; %array of asset weights estimated at different steps in the loop
            COVM   = [COVM,CovMat]; %array of time-varying covariance matrices estimated at different steps in the loop
            INDEX  = [INDEX,ind];
            EstPar = [EstPar;estPar]
        
    end
    
    if VaR < -0.1*capp
        VaR = 0;
    end
    
    VaRQ   = [VaRQ,VaR] %Value-at-Risk vector
    wshift = wshift + 1
    
end

CAP = [1,cappp]; %final cumulative return vector for Strategy 1


%STRATEGY 3 ("NAIVE") CUMULATIVE RETURN CALCULATION LOOP (practically all calculations
%are the same with some exceptions pointed out below)

clear cap capp cappp VaR
cap{wwidth}  = 1; 
capp(wwidth) = 1;
cappp        = [];
wshift       = 0;
VaRN         = [];
NUMLAM       = [];


for l = wwidth:size(FUNDS,1)
    
    if IND(l) > quantile(IND(1+wshift:l,:),0.50)
        cap{l}   = sum(cell2mat(cap(l)));
        MeanRet  = mean(IND(1:l,:));
        sigma    = std(IND(1:l,:));
        skewn    = skewness(IND(1:l,:));
        kurt     = kurtosis(IND(1:l,:));
        VaR      = sum(cell2mat(cap(l)))*( - (z + (1/6)*(z^2-1)*skewn+(1/24)*(z^3-3*z)*(kurt-3)-(1/36)*(2*z^3-5*z)*skewn^2)*sigma);
        cap{l+1} = sum(cell2mat(cap(l)).*(1 + IND(l)));
        capp     = cap{l+1}
        cappp    = [cappp,capp]
    else
            X     = XRET(1+wshift:l,:);
            Y     = YRET(1+wshift:l,:);
            [n,k] = size(X);
    
            coeff = [];
            SIC   = [];
            for j = 1:length(lambda)
    
                beta = [];
                sic  = [];
                for i = 1:length(tau)
                    c          = [tau(i).*ones(1,n),(1-tau(i)).*ones(1,n),lambda(j).*ones(1,k),zeros(1,k)];
                    A          = [eye(n),-eye(n),zeros(n,k),X];
                    A          = sparse(A);
                    B          =  [-eye(k,n),zeros(k,n),zeros(k,k),zeros(k,k); 
                                    zeros(k,n),-eye(k,n),zeros(k,k),zeros(k,k);
                                    zeros(k,n),zeros(k,n),-eye(k,k),eye(k,k);
                                    zeros(k,n),zeros(k,n),-eye(k,k),-eye(k,k)
                                    zeros(k,n),zeros(k,n),zeros(k,k),eye(k,k)];
                    B          = sparse(B);

                    q          = [zeros(1,k),zeros(1,k),zeros(1,k),zeros(1,k),zeros(1,k)]';
                    b          = Y;
                    lb         = [zeros(1,n),zeros(1,n),-Inf.*ones(1,k),-Inf.*ones(1,k)];
                    ub         = [Inf.*ones(1,n),Inf.*ones(1,n),Inf.*ones(1,k),zeros(1,k)];
                    options    = optimset('LargeScale','on','MaxIter',2000);
                    bb         = linprog(c,B,q,A,b,lb,ub,[],options);
                    bbeta      = bb(length(bb) - k + 1:end);
                    df         = length(bbeta(bbeta ~= 0));
                    qrLoss     = sum(qrRho(Y - X*bbeta,tau(i)));
                    sicc       = n*log(n^(-1)*qrLoss) + log(n)*df;
                    beta       = [beta,bbeta];
                    sic        = [sic,sicc];
                end
                coeff = [coeff,beta];
                SIC   = [SIC;sic];
            end
            sicind05    = 1:5:length(tau)*length(lambda)-4;
            sicind15    = 2:5:length(tau)*length(lambda)-3;
            sicind25    = 3:5:length(tau)*length(lambda)-2;
            sicind35    = 4:5:length(tau)*length(lambda)-1;
            sicind50    = 5:5:length(tau)*length(lambda)-0;

            beta05      = coeff(:,sicind05);
            beta15      = coeff(:,sicind15);
            beta25      = coeff(:,sicind25);
            beta35      = coeff(:,sicind35);
            beta50      = coeff(:,sicind50);

            sic05       = find(SIC(:,1) == min(SIC(:,1)));
            sic15       = find(SIC(:,2) == min(SIC(:,2)));
            sic25       = find(SIC(:,3) == min(SIC(:,3)));
            sic35       = find(SIC(:,4) == min(SIC(:,4)));
            sic50       = find(SIC(:,5) == min(SIC(:,5)));
            sicall      = [sic05, sic15, sic25, sic35, sic50];
            NUMLAM      = [NUMLAM; sicall]

            Beta05      = beta05(:,sic05);
            Beta15      = beta15(:,sic15);
            Beta25      = beta25(:,sic25);
            Beta35      = beta35(:,sic35);
            Beta50      = beta50(:,sic50);

            BetaAll     = [Beta05,Beta15,Beta25,Beta35,Beta50];

            clear XWW
            XWW = {};
            for i = 1:length(tau)
                XW = [];
                for j = 1:n
                    XWi = X(j,:).*max(abs(BetaAll(:,i)'),1/sqrt(n));
                    XW  = [XW;XWi];
                end
                XWW = [XWW,XW];
            end
    
            BetaLam = round(BetaAll.*(10^num_digw))./(10^num_digw);

            Lambda  = 0.25*sqrt(sum(abs(BetaLam) > 0))*log(k)*(log(n))^(0.1/2);
            COEFF   = [];
    
            for i = 1:length(Lambda)
                c       = [tau(i).*ones(1,n),(1-tau(i)).*ones(1,n),Lambda(i).*ones(1,k),zeros(1,k)];
                A       = [eye(n),-eye(n),eye(n,k),XWW{i}];
                B       =  [-eye(n),zeros(n),zeros(n,k),zeros(n,k); 
                             zeros(n),-eye(n),zeros(n,k),zeros(n,k);
                             zeros(n),zeros(n),-eye(n,k),eye(n,k);
                             zeros(n),zeros(n),-eye(n,k),-eye(n,k)
                             zeros(n),zeros(n),zeros(n,k),eye(n,k)];
                q       = [zeros(1,n),zeros(1,n),zeros(1,n),zeros(1,n),zeros(1,n)]';
                b       = Y;
                lb      = [zeros(1,n),zeros(1,n),-Inf.*ones(1,k),-Inf.*ones(1,k)];
                ub      = [Inf.*ones(1,n),Inf.*ones(1,n),Inf.*ones(1,k),zeros(1,k)];
                options = optimset('LargeScale','on','MaxIter',2000);
                bb      = linprog(c,B,q,A,b,lb,ub,[],options);
                COE     = bb(length(bb) - k + 1:end);
                COEFF   = [COEFF,COE];   
            end

            Coeff   = COEFF.*max(abs(BetaLam),1/sqrt(n));
    
            BETAFIN = round(Coeff.*(10^num_digw))./(10^num_digw);
            options = optimset('Algorithm','active-set');

            if IND(l) <= quantile(IND(1+wshift:l,:),0.05)
                ind     = find(BETAFIN(:,1) ~= 0);
                RetMat  = FUNDS(1:l,ind);
                wgt     = ones(1,size(RetMat,2))./size(RetMat,2);
                MeanRet = mean(RetMat)'; %calculating the mean return vector
                P       = RetMat;
                
                if size(P,2) == 0
                    cap{l}   = sum(cell2mat(cap(l)));
                    sigma    = std(IND(1:l,:)); %calculating the S&P500 position volatility
                    skewn    = skewness(IND(1:l,:)); %calculating the S&P500 position skewness
                    kurt     = kurtosis(IND(1:l,:)); %calculating the S&P500 position kurtosis
                    %calculating the S&P500 position risk
                    VaR      = sum(cell2mat(cap(l)))*( - (z + (1/6)*(z^2-1)*skewn+(1/24)*(z^3-3*z)*(kurt-3)-(1/36)*(2*z^3-5*z)*skewn^2)*sigma);
                    %portfolio value appreciation
                    cap{l+1} = sum(cell2mat(cap(l)).*(1 + IND(l)));
                    capp     = cap{l+1};
                    cappp    = [cappp,capp]
                    VaRN     = [VaRN,VaR] 
                    wshift   = wshift + 1
                    continue                     
                end
                
                M2=cov(P);
                M3=[];
                for i=1:size(P,2)
                    S=[];
                     for j=1:size(P,2)
                        for k=1:size(P,2)
                            u=0;
                            for t=1:size(P,1) 
                                u=u+((P(t,i))*(P(t,j)) ...
                                    *(P(t,k)));
                            end
                            S(j,k)=u/(size(P,1)); 
                        end
                     end
                    M3=[M3 S];
                end
                M4=[];
                for i=1:size(P,2)
                    for j=1:size(P,2)
                        S=[];
                        for k=1:size(P,2)
                            for m=1:size(P,2)
                                u=0;
                                for t=1:size(P,1)
                                    u=u+((P(t,i))*(P(t,j))* ...
                                        (P(t,k))*(P(t,m)));
                                end
                                S(k,m)=u/(size(P,1));
                            end
                        end
                        M4=[M4 S];
                    end
                end
                cap{l} = (repmat(sum(cell2mat(cap(l)))/(length(ind)),1,length(ind))); %investing into all chosen hedge funds with equal weights
                %Cornish-Fisher VaR calculation with constant covariance
                %structure and equal weights (without any optimization)
   
                %wgt*MeanRet
                VaR      = sum(cell2mat(cap(l)))*( - (z+(z^2-1)*(((1/((wgt*M2*wgt')^(3/2)))*(wgt*M3*kron(wgt',wgt')-3*(wgt*M2*wgt' + (wgt*MeanRet)^2)*...
                          (wgt*MeanRet)+2*((wgt*MeanRet)^3)))/6) + (z^3-3*z)*((((1/((wgt*M2*wgt')^2))*(wgt*M4*kron(kron(wgt',wgt'),wgt')-4*wgt*M3*kron(wgt',wgt')*...
                          (wgt*MeanRet)+6*(wgt*M2*wgt' + (wgt*MeanRet)^2)*(wgt*MeanRet)^2+3*(wgt*MeanRet)^4))-3)/24) - (2*z^3-5*z)*(((((1/((wgt*M2*wgt')^(3/2)))*...
                          (wgt*M3*kron(wgt',wgt')-3*(wgt*M2*wgt' + (wgt*MeanRet)^2)*(wgt*MeanRet)+2*((wgt*MeanRet)^3))))^2)/36))*(sqrt(wgt*M2*wgt')));
                cap{l+1} = sum(cell2mat(cap(l)).*(1 + FUNDS(l,ind)));
                capp     = cap{l+1};
                cappp    = [cappp,capp] 
            elseif IND(l) <= quantile(IND(1+wshift:l,:),0.15)&& IND(l) > quantile(IND(1+wshift:l,:),0.05)
                ind      = find(BETAFIN(:,2) ~= 0);
                RetMat   = FUNDS(1:l,ind);
                wgt      = ones(1,size(RetMat,2))./size(RetMat,2);
                MeanRet  = mean(RetMat)';
                P        = RetMat;
                
                if size(P,2) == 0
                    cap{l}   = sum(cell2mat(cap(l)));
                    sigma    = std(IND(1:l,:)); %calculating the S&P500 position volatility
                    skewn    = skewness(IND(1:l,:)); %calculating the S&P500 position skewness
                    kurt     = kurtosis(IND(1:l,:)); %calculating the S&P500 position kurtosis
                    %calculating the S&P500 position risk
                    VaR      = sum(cell2mat(cap(l)))*( - (z + (1/6)*(z^2-1)*skewn+(1/24)*(z^3-3*z)*(kurt-3)-(1/36)*(2*z^3-5*z)*skewn^2)*sigma);
                    %portfolio value appreciation
                    cap{l+1} = sum(cell2mat(cap(l)).*(1 + IND(l)));
                    capp     = cap{l+1};
                    cappp    = [cappp,capp]
                    VaRN     = [VaRN,VaR] 
                    wshift   = wshift + 1
                    continue                     
                end
                
                M2=cov(P);
                M3=[];
                for i=1:size(P,2)
                    S=[];
                     for j=1:size(P,2)
                        for k=1:size(P,2)
                            u=0;
                            for t=1:size(P,1) 
                                u=u+((P(t,i))*(P(t,j)) ...
                                    *(P(t,k)));
                            end
                            S(j,k)=u/(size(P,1)); 
                        end
                     end
                    M3=[M3 S];
                end
                M4=[];
                for i=1:size(P,2)
                    for j=1:size(P,2)
                        S=[];
                        for k=1:size(P,2)
                            for m=1:size(P,2)
                                u=0;
                                for t=1:size(P,1)
                                    u=u+((P(t,i))*(P(t,j))* ...
                                        (P(t,k))*(P(t,m)));
                                end
                                S(k,m)=u/(size(P,1));
                            end
                        end
                        M4=[M4 S];
                    end
                end
                cap{l}   = (repmat(sum(cell2mat(cap(l)))/(length(ind)),1,length(ind)));
                %wgt*MeanRet
                VaR      = sum(cell2mat(cap(l)))*( - (z+(z^2-1)*(((1/((wgt*M2*wgt')^(3/2)))*(wgt*M3*kron(wgt',wgt')-3*(wgt*M2*wgt' + (wgt*MeanRet)^2)*...
                          (wgt*MeanRet)+2*((wgt*MeanRet)^3)))/6) + (z^3-3*z)*((((1/((wgt*M2*wgt')^2))*(wgt*M4*kron(kron(wgt',wgt'),wgt')-4*wgt*M3*kron(wgt',wgt')*...
                          (wgt*MeanRet)+6*(wgt*M2*wgt' + (wgt*MeanRet)^2)*(wgt*MeanRet)^2+3*(wgt*MeanRet)^4))-3)/24) - (2*z^3-5*z)*(((((1/((wgt*M2*wgt')^(3/2)))*...
                          (wgt*M3*kron(wgt',wgt')-3*(wgt*M2*wgt' + (wgt*MeanRet)^2)*(wgt*MeanRet)+2*((wgt*MeanRet)^3))))^2)/36))*(sqrt(wgt*M2*wgt')));
                cap{l+1} = sum(cell2mat(cap(l)).*(1 + FUNDS(l,ind)));
                capp     = cap{l+1};
                cappp    = [cappp,capp] 
            elseif IND(l) <= quantile(IND(1+wshift:l,:),0.25)&& IND(l) > quantile(IND(1+wshift:l,:),0.15)
                ind     = find(BETAFIN(:,3) ~= 0);
                RetMat  = FUNDS(1:l,ind);
                wgt     = ones(1,size(RetMat,2))./size(RetMat,2);
                MeanRet = mean(RetMat)';
                P       = RetMat;
                
                if size(P,2) == 0
                    cap{l}   = sum(cell2mat(cap(l)));
                    sigma    = std(IND(1:l,:)); %calculating the S&P500 position volatility
                    skewn    = skewness(IND(1:l,:)); %calculating the S&P500 position skewness
                    kurt     = kurtosis(IND(1:l,:)); %calculating the S&P500 position kurtosis
                    %calculating the S&P500 position risk
                    VaR      = sum(cell2mat(cap(l)))*( - (z + (1/6)*(z^2-1)*skewn+(1/24)*(z^3-3*z)*(kurt-3)-(1/36)*(2*z^3-5*z)*skewn^2)*sigma);
                    %portfolio value appreciation
                    cap{l+1} = sum(cell2mat(cap(l)).*(1 + IND(l)));
                    capp     = cap{l+1};
                    cappp    = [cappp,capp]
                    VaRN     = [VaRN,VaR] 
                    wshift   = wshift + 1
                    continue                     
                end
                
                M2=cov(P);
                M3=[];
                for i=1:size(P,2)
                    S=[];
                     for j=1:size(P,2)
                        for k=1:size(P,2)
                            u=0;
                            for t=1:size(P,1) 
                                u=u+((P(t,i))*(P(t,j)) ...
                                    *(P(t,k)));
                            end
                            S(j,k)=u/(size(P,1)); 
                        end
                     end
                    M3=[M3 S];
                end
                M4=[];
                for i=1:size(P,2)
                    for j=1:size(P,2)
                        S=[];
                        for k=1:size(P,2)
                            for m=1:size(P,2)
                                u=0;
                                for t=1:size(P,1)
                                    u=u+((P(t,i))*(P(t,j))* ...
                                        (P(t,k))*(P(t,m)));
                                end
                                S(k,m)=u/(size(P,1));
                            end
                        end
                        M4=[M4 S];
                    end
                end
                cap{l}   = (repmat(sum(cell2mat(cap(l)))/(length(ind)),1,length(ind)));
                %wgt*MeanRet
                VaR      = sum(cell2mat(cap(l)))*( - (z+(z^2-1)*(((1/((wgt*M2*wgt')^(3/2)))*(wgt*M3*kron(wgt',wgt')-3*(wgt*M2*wgt' + (wgt*MeanRet)^2)*...
                          (wgt*MeanRet)+2*((wgt*MeanRet)^3)))/6) + (z^3-3*z)*((((1/((wgt*M2*wgt')^2))*(wgt*M4*kron(kron(wgt',wgt'),wgt')-4*wgt*M3*kron(wgt',wgt')*...
                          (wgt*MeanRet)+6*(wgt*M2*wgt' + (wgt*MeanRet)^2)*(wgt*MeanRet)^2+3*(wgt*MeanRet)^4))-3)/24) - (2*z^3-5*z)*(((((1/((wgt*M2*wgt')^(3/2)))*...
                          (wgt*M3*kron(wgt',wgt')-3*(wgt*M2*wgt' + (wgt*MeanRet)^2)*(wgt*MeanRet)+2*((wgt*MeanRet)^3))))^2)/36))*(sqrt(wgt*M2*wgt')));
                cap{l+1} = sum(cell2mat(cap(l)).*(1 + FUNDS(l,ind)));
                capp     = cap{l+1};
                cappp    = [cappp,capp] 
            elseif IND(l) <= quantile(IND(1+wshift:l,:),0.35)&& IND(l) > quantile(IND(1+wshift:l,:),0.25)
                ind      = find(BETAFIN(:,4) ~= 0);
                RetMat   = FUNDS(1:l,ind);
                wgt      = ones(1,size(RetMat,2))./size(RetMat,2);
                MeanRet  = mean(RetMat)';
                P        = RetMat;
                
                if size(P,2) == 0
                    cap{l}   = sum(cell2mat(cap(l)));
                    MeanRet  = mean(IND(1:l,:)); %calculating the mean return vector
                    sigma    = std(IND(1:l,:)); %calculating the S&P500 position volatility
                    skewn    = skewness(IND(1:l,:)); %calculating the S&P500 position skewness
                    kurt     = kurtosis(IND(1:l,:)); %calculating the S&P500 position kurtosis
                    %calculating the S&P500 position risk
                    VaR      = sum(cell2mat(cap(l)))*( - (z + (1/6)*(z^2-1)*skewn+(1/24)*(z^3-3*z)*(kurt-3)-(1/36)*(2*z^3-5*z)*skewn^2)*sigma);
                    %portfolio value appreciation
                    cap{l+1} = sum(cell2mat(cap(l)).*(1 + IND(l)));
                    capp     = cap{l+1};
                    cappp    = [cappp,capp]
                    VaRN     = [VaRN,VaR] 
                    wshift   = wshift + 1
                    continue                     
                end
                
                M2=cov(P);
                M3=[];
                for i=1:size(P,2)
                    S=[];
                     for j=1:size(P,2)
                        for k=1:size(P,2)
                            u=0;
                            for t=1:size(P,1) 
                                u=u+((P(t,i))*(P(t,j)) ...
                                    *(P(t,k)));
                            end
                            S(j,k)=u/(size(P,1)); 
                        end
                     end
                    M3=[M3 S];
                end
                M4=[];
                for i=1:size(P,2)
                    for j=1:size(P,2)
                        S=[];
                        for k=1:size(P,2)
                            for m=1:size(P,2)
                                u=0;
                                for t=1:size(P,1)
                                    u=u+((P(t,i))*(P(t,j))* ...
                                        (P(t,k))*(P(t,m)));
                                end
                                S(k,m)=u/(size(P,1));
                            end
                        end
                        M4=[M4 S];
                    end
                end
                cap{l}   = (repmat(sum(cell2mat(cap(l)))/(length(ind)),1,length(ind)));
                %wgt*MeanRet
                VaR      = sum(cell2mat(cap(l)))*( - (z+(z^2-1)*(((1/((wgt*M2*wgt')^(3/2)))*(wgt*M3*kron(wgt',wgt')-3*(wgt*M2*wgt' + (wgt*MeanRet)^2)*...
                          (wgt*MeanRet)+2*((wgt*MeanRet)^3)))/6) + (z^3-3*z)*((((1/((wgt*M2*wgt')^2))*(wgt*M4*kron(kron(wgt',wgt'),wgt')-4*wgt*M3*kron(wgt',wgt')*...
                          (wgt*MeanRet)+6*(wgt*M2*wgt' + (wgt*MeanRet)^2)*(wgt*MeanRet)^2+3*(wgt*MeanRet)^4))-3)/24) - (2*z^3-5*z)*(((((1/((wgt*M2*wgt')^(3/2)))*...
                          (wgt*M3*kron(wgt',wgt')-3*(wgt*M2*wgt' + (wgt*MeanRet)^2)*(wgt*MeanRet)+2*((wgt*MeanRet)^3))))^2)/36))*(sqrt(wgt*M2*wgt')));
                cap{l+1} = sum(cell2mat(cap(l)).*(1 + FUNDS(l,ind)));
                capp     = cap{l+1};
                cappp    = [cappp,capp]
            else
                ind      = find(BETAFIN(:,5) ~= 0);
                RetMat   = FUNDS(1:l,ind);                            
                wgt      = ones(1,size(RetMat,2))./size(RetMat,2);
                MeanRet  = mean(RetMat)';
                P        = RetMat;
                
                if size(P,2) == 0
                    cap{l}   = sum(cell2mat(cap(l)));
                    MeanRet  = mean(IND(1:l,:)); %calculating the mean return vector
                    sigma    = std(IND(1:l,:)); %calculating the S&P500 position volatility
                    skewn    = skewness(IND(1:l,:)); %calculating the S&P500 position skewness
                    kurt     = kurtosis(IND(1:l,:)); %calculating the S&P500 position kurtosis
                    %calculating the S&P500 position risk
                    VaR      = sum(cell2mat(cap(l)))*( - (z + (1/6)*(z^2-1)*skewn+(1/24)*(z^3-3*z)*(kurt-3)-(1/36)*(2*z^3-5*z)*skewn^2)*sigma);
                    %portfolio value appreciation
                    cap{l+1} = sum(cell2mat(cap(l)).*(1 + IND(l)));
                    capp     = cap{l+1};
                    cappp    = [cappp,capp]
                    VaRN     = [VaRN,VaR] 
                    wshift   = wshift + 1
                    continue                     
                end
                
                M2=cov(P);
                M3=[];
                for i=1:size(P,2)
                    S=[];
                     for j=1:size(P,2)
                        for k=1:size(P,2)
                            u=0;
                            for t=1:size(P,1) 
                                u=u+((P(t,i))*(P(t,j)) ...
                                    *(P(t,k)));
                            end
                            S(j,k)=u/(size(P,1)); 
                        end
                     end
                    M3=[M3 S];
                end
                M4=[];
                for i=1:size(P,2)
                    for j=1:size(P,2)
                        S=[];
                        for k=1:size(P,2)
                            for m=1:size(P,2)
                                u=0;
                                for t=1:size(P,1)
                                    u=u+((P(t,i))*(P(t,j))* ...
                                        (P(t,k))*(P(t,m)));
                                end
                                S(k,m)=u/(size(P,1));
                            end
                        end
                        M4=[M4 S];
                    end
                end
                cap{l}   = (repmat(sum(cell2mat(cap(l)))/(length(ind)),1,length(ind)));
                %wgt*MeanRet
                VaR      = sum(cell2mat(cap(l)))*( - (z+(z^2-1)*(((1/((wgt*M2*wgt')^(3/2)))*(wgt*M3*kron(wgt',wgt')-3*(wgt*M2*wgt' + (wgt*MeanRet)^2)*...
                          (wgt*MeanRet)+2*((wgt*MeanRet)^3)))/6) + (z^3-3*z)*((((1/((wgt*M2*wgt')^2))*(wgt*M4*kron(kron(wgt',wgt'),wgt')-4*wgt*M3*kron(wgt',wgt')*...
                          (wgt*MeanRet)+6*(wgt*M2*wgt' + (wgt*MeanRet)^2)*(wgt*MeanRet)^2+3*(wgt*MeanRet)^4))-3)/24) - (2*z^3-5*z)*(((((1/((wgt*M2*wgt')^(3/2)))*...
                          (wgt*M3*kron(wgt',wgt')-3*(wgt*M2*wgt' + (wgt*MeanRet)^2)*(wgt*MeanRet)+2*((wgt*MeanRet)^3))))^2)/36))*(sqrt(wgt*M2*wgt')));
                cap{l+1} = sum(cell2mat(cap(l)).*(1 + FUNDS(l,ind)));
                capp     = cap{l+1};
                cappp    = [cappp,capp]        
            end
                
    end
    VaRN   = [VaRN,VaR] %VaR calculation
    wshift = wshift + 1
    
end

  CAPN   = [1,cappp]; %final cumulative return vector for Strategy 3

  %creating input data for the number of selected variables
  %plots
  indc05 = 1:5:size(BETA,2)-4;
  indc15 = 2:5:size(BETA,2)-3;
  indc25 = 3:5:size(BETA,2)-2;
  indc35 = 4:5:size(BETA,2)-1;
  indc50 = 5:5:size(BETA,2)-0;
  
  BETA05 = BETA(:,indc05);
  BETA15 = BETA(:,indc15);
  BETA25 = BETA(:,indc25);
  BETA35 = BETA(:,indc35);
  BETA50 = BETA(:,indc50);
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %EST. BETA PLOT FOR TAU=0.05
%   for i = 1:size(BETA05,2)
%       h = figure; 
%       hold on
%       plot(-BETA05(:,i),'.','MarkerSize',20,'Color','r');
%       title()
%       print (h, '-r600', '-depsc', strcat('fbeta',num2str(i)));
%       eps2pdf(strcat('fbeta',num2str(i),'.eps'), strcat('fbeta',num2str(i),'.pdf'), 1, 0)
%       hold off
%   end
%   %EST. BETA PLOT FOR TAU=0.15
%   for i = 1:size(BETA15,2)
%       h = figure; 
%       hold on
%       plot(-BETA15(:,i),'.','MarkerSize',20,'Color','r');
%       print (h, '-r600', '-depsc', strcat('ftbeta',num2str(i)));
%       eps2pdf(strcat('ftbeta',num2str(i),'.eps'), strcat('ftbeta',num2str(i),'.pdf'), 1, 0)
%       hold off
%   end
%   %EST. BETA PLOT FOR TAU=0.25
%   for i = 1:size(BETA25,2)
%       h = figure; 
%       hold on
%       plot(-BETA25(:,i),'.','MarkerSize',20,'Color','r');
%       print (h, '-r600', '-depsc', strcat('tfbeta',num2str(i)));
%       eps2pdf(strcat('tfbeta',num2str(i),'.eps'), strcat('tfbeta',num2str(i),'.pdf'), 1, 0)
%       hold off
%   end
%   %EST. BETA PLOT FOR TAU=0.35
%   for i = 1:size(BETA35,2)
%       h = figure; 
%       hold on
%       plot(-BETA35(:,i),'.','MarkerSize',20,'Color','r');
%       print (h, '-r600', '-depsc', strcat('thbeta',num2str(i)));
%       eps2pdf(strcat('thbeta',num2str(i),'.eps'), strcat('thbeta',num2str(i),'.pdf'), 1, 0)
%       hold off
%   end
%   %EST. BETA PLOT FOR TAU=0.50
%   for i = 1:size(BETA50,2)
%       h = figure; 
%       hold on
%       plot(-BETA50(:,i),'.','MarkerSize',20,'Color','r');
%       print (h, '-r600', '-depsc', strcat('ffbeta',num2str(i)));
%       eps2pdf(strcat('ffbeta',num2str(i),'.eps'), strcat('ffbeta',num2str(i),'.pdf'), 1, 0)
%       hold off
%   end
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
  for i = 1:size(BETA05,2)
      BETA05q(i) = sum(BETA05(:,i) ~= 0);
  end
  
  for i = 1:size(BETA15,2)
      BETA15q(i) = sum(BETA15(:,i) ~= 0);
  end
  
  for i = 1:size(BETA25,2)
      BETA25q(i) = sum(BETA25(:,i) ~= 0);
  end
  
  for i = 1:size(BETA35,2)
      BETA35q(i) = sum(BETA35(:,i) ~= 0);
  end
  
  for i = 1:size(BETA50,2)
      BETA50q(i) = sum(BETA50(:,i) ~= 0);
  end
  
  %FIGURE 12 replication
  nbc = 13;  
  figure, hold('on')
  subplot(2,2,1)
  
  hist(BETA05q,1:1:nbc)
  title('tau = 0.05')
  subplot(2,2,2)
  
  hist(BETA15q,1:1:nbc)
  title('tau = 0.15')
  subplot(2,2,3)
 
  hist(BETA25q,1:1:nbc)
  title('tau = 0.25')
  subplot(2,2,4)
  
  hist(BETA35q,1:1:nbc)
  title('tau = 0.35')
  hold off
  
  %creating input data for the frequency of selected hedge funds
  %plots 
  for i = 1:size(BETA05,1)
      BETA05f(i) = sum(BETA05(i,:) ~= 0);
  end
  
  for i = 1:size(BETA15,1)
      BETA15f(i) = sum(BETA15(i,:) ~= 0);
  end
  
  for i = 1:size(BETA25,1)
      BETA25f(i) = sum(BETA25(i,:) ~= 0);
  end
  
  for i = 1:size(BETA35,1)
      BETA35f(i) = sum(BETA35(i,:) ~= 0);
  end
  
  for i = 1:size(BETA50,1)
      BETA50f(i) = sum(BETA50(i,:) ~= 0);
  end
  
  
  %FIGURE 13 replication
  figure, hold('on')
  subplot(2,2,1)
  
  bar(1:1:size(BETA05,1),BETA05f,2)
  title('tau = 0.05')
  subplot(2,2,2)
  
  bar(1:1:size(BETA05,1),BETA15f,2)
  title('tau = 0.15')
  subplot(2,2,3)
 
  bar(1:1:size(BETA05,1),BETA25f,2)
  title('tau = 0.25')
  subplot(2,2,4)
  
  bar(1:1:size(BETA05,1),BETA35f,2)
  title('tau = 0.35')
  hold off
  
  %finding most influential hedge funds (Tables 8-10)
  [freq05, idx05] = sort(BETA05f,'descend');
  [freq15, idx15] = sort(BETA15f,'descend');
  [freq25, idx25] = sort(BETA25f,'descend');
  [freq35, idx35] = sort(BETA35f,'descend');
  [freq50, idx50] = sort(BETA50f,'descend');

disp('TAU05')
[freq05(1:10);idx05(1:10)]
disp('TAU15')
[freq15(1:10);idx15(1:10)]
disp('TAU35')
[freq35(1:10);idx35(1:10)]

hf_bloom_tick{idx05(1:9)}

hf_bloom_tick{idx15(1:7)}
hf_bloom_tick{idx35(1:7)}

% hedfhdnames_short{idx05(1:6)}
% hedfhdnames_short{idx15(1:6)}
% hedfhdnames_short{idx35(1:3)}


  %FIGURE 11 replication
  lamdate = [];
  wshift = 0;
  for i = wwidth:size(FUNDS,1)
    
    if IND(i) <= quantile(IND(1+wshift:i,:),0.50)
        
        lamdate = [lamdate,i];
    end
    wshift = wshift+1
  end
  LamDate = Date(lamdate);
  
  figure
  subplot(2,2,1)
  
  plot(LamDate,LAMBDA(:,numlam(2:end,1)),'LineWidth',2)
  hold on
  datetick('x')
  title('tau = 0.05')
  subplot(2,2,2)
  
  plot(LamDate,lambda(:,NUMLAM(2:end,2)),'LineWidth',2)
  hold on
  datetick('x')
  title('tau = 0.15')
  subplot(2,2,3)
 
  plot(LamDate,lambda(:,NUMLAM(2:end,3)),'LineWidth',2)
  hold on
  datetick('x')
  title('tau = 0.25')
  subplot(2,2,4)
  
  plot(LamDate,lambda(:,NUMLAM(2:end,4)),'LineWidth',2)
  hold on
  datetick('x')
  title('tau = 0.35')
  hold off
  
   
%STRATEGY 4 (ORTHOGONAL GARCH) CUMULATIVE RETURN CALCULATION LOOP
OGRETI        = HFBloomRet(1:end,2:end-1);
clear capit OGCAPIT
capit(wwidth) = 1;
OGCAPIT       = [];
VaROG         = [];
stshift       = 0;
num_dig       = 4;
varthresh     = 95; %percent of explained variance in orthogonal GARCH
NUMFAC        = [];
options       = optimset('Algorithm','active-set');

for i = wwidth:size(OGRETI,1)
    if IND(i) > quantile(IND(1+stshift:i,:),0.50)%this part as before
        MeanRet    = mean(IND(1:i,:));
        sigma      = std(IND(1:i,:));
        skewn      = skewness(IND(1:i,:));
        kurt       = kurtosis(IND(1:i,:));
        iVaR       = sum(capit(i))*( - (z + (1/6)*(z^2-1)*skewn+(1/24)*(z^3-3*z)*(kurt-3)-(1/36)*(2*z^3-5*z)*skewn^2)*sigma);
        capit(i+1) = sum(capit(i).*(1 + IND(i)));
        caplast    = capit(end);
        OGCAPIT    = [OGCAPIT,caplast];
    else
        XI              = OGRETI(1:i,:);
        [Ht,numfactors] = ogarch(XI,1,1,1,1,varthresh); %calculating the time-varying covariance matrix via the orthogonal GARCH
        %setting up the variance-covariance VaR optimization procedure
        w0              = ones(1,size(XI,2))./size(XI,2);
        MeanRet         = mean(XI)';
        ub              = ones(length(w0),1);
        lb              = zeros(length(w0),1);
        Aeq             = ones(1,length(w0));
        beq             = 1;
        AA              = -MeanRet';
        bb              = -quantile(MeanRet,TargRet);
        [wgt,iVaR]      = fmincon(@(w)sum(capit(i))*( - z*(sqrt(w*Ht*w'))),w0,AA,bb,Aeq,beq,lb,ub,[],options);
        wgt             = round(wgt.*(10^num_dig))./(10^num_dig);
        %portfolio appreciation
        capit(i+1)      = sum(capit(i).*wgt'.*((1 + OGRETI(i,:))'));
        caplast         = capit(end);
        OGCAPIT         = [OGCAPIT,caplast];
        NUMFAC          = [NUMFAC,numfactors];
    end
    VaROG   = [VaROG,iVaR]; %calculating the VaR vector
    stshift = stshift + 1
end
   
OGCAPITST = [1,OGCAPIT]; %final cumulative return vector for Strategy 4 


%STRATEGY 2 (S&P BUY-AND-HOLD)
clear INDCAPIT INDCAP
INDCAP(wwidth) = 1;
INDCAPIT       = [];
ishift         = 0;
VaRI           = [];
for i = wwidth:size(IND,1)
    
    MeanRet     = mean(IND(1:i,:));
    sigma       = std(IND(1:i,:));
    skewn       = skewness(IND(1:i,:));
    kurt        = kurtosis(IND(1:i,:));
    %calculating the S&P500 position VaR
    %MeanRet
    VaR         = sum(INDCAP(i))*( - (z + (1/6)*(z^2-1)*skewn+(1/24)*(z^3-3*z)*(kurt-3)-(1/36)*(2*z^3-5*z)*skewn^2)*sigma);
    INDCAP(i+1) = INDCAP(i)*(1+IND(i));
    indcaplast  = INDCAP(i+1);
    INDCAPIT    = [INDCAPIT,indcaplast];
    
    VaRI        = [VaRI,VaR];
    ishift      = ishift + 1
end

INDCAPITST = [1,INDCAPIT]; %final cumulative return vector for Strategy 2

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Transaction cost calculation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
perc      = 0.01;
count     = 0;
wshift    = 0;
TransCost = zeros(1,length(CAP)-1);
%ii = zeros(1,length(CAP)-1);
ii        = [];
for i = wwidth:size(FUNDS,1)
    if IND(i) <= quantile(IND(1+wshift:i,:),0.50)
        TransCost(i-wwidth+2) = perc*CAP(i-wwidth+2);
        count                 = count+1
        %ii(i-wwidth+2) = i-wwidth+2;
        ii                    = [ii,i];
    else
        TransCost(i-wwidth+2) = TransCost(i-wwidth+1);
    end
    wshift = wshift + 1;
end
idate = ii
ii    = ii-wwidth+1
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
DateP = Date(ii);

%FIGURE 8 REPLICATION (strategies' comparison plot + TRANSACTION COST)
% figure
% sub1 = subplot(2,1,1);
% pos1 = get(sub1, 'Position')
% set(sub1, 'Position', pos1 + [0.00 -0.10 0 0.10])
% plot(Date(wwidth:end,:),INDCAPITST(1:end),'b','LineWidth',4)
% hold on
% grid on
% datetick('x')
% xlabel('Time'), ylabel('Cumulative Return'), title('Comparison of Strategies')
% plot(Date(wwidth:end,:),OGCAPITST(1:end),'m','LineWidth',4)
% %col = [0 0.5 0];
% plot(Date(wwidth:end,:),CAPN(1:end),'Color','g','LineWidth',4)
% plot(Date(wwidth:end,:),CAP(1:end),'r','LineWidth',4)
% sub2 = subplot(2,1,2);
% pos2 = get(sub2, 'Position')
% set(sub2, 'Position', pos2 - [0.00 0.02 0 0.10])
% plot(Date(wwidth:end,:),TransCost,'r','LineWidth',4)
% hold on
% grid on
% datetick('x')
% hold on
% xlabel('Time'), ylabel('Cost'),title('Cumulative Transaction Cost')
% hold on
% plot(Date(idate),TransCost(ii),'.','Color','b','MarkerSize',20)
% datetick('x')
% hold off

%FIGURE WITH CUMULATIVE RETURN FOR ALL STRATEGIES
figure
plot(Date(wwidth-1:end-1,:),INDCAPITST(1:end),'b','LineWidth',4)
hold on
grid on
datetick('x')
xlabel('Time'), ylabel('Cumulative Return'), title('Comparison of Strategies')
plot(Date(wwidth-1:end-1,:),OGCAPITST(1:end),'m','LineWidth',4)
%col = [0 0.5 0];
plot(Date(wwidth-1:end-1,:),CAPN(1:end),'Color','g','LineWidth',4)
plot(Date(wwidth-1:end-1,:),CAP(1:end),'r','LineWidth',4)
hold off


%FIGURE WITH TRANSACTION COST FOR ALL STRATEGIES
figure
plot(Date(wwidth:end-1,:),TransCost(1:end-1),'r','LineWidth',4)
hold on
grid on
datetick('x')
hold on
xlabel('Time'), ylabel('Cost'),title('Cumulative Transaction Cost')
hold on
plot(Date(idate),TransCost(ii),'.','Color','b','MarkerSize',20)
datetick('x')
hold off


%FIGURE 10 REPLICATION (risks' comparison plot)
figure
subplot(2,1,1)
axis([0 21 -0.40 0.70])
plot(Date(wwidth:end-1,:),VaRQ,'r','LineWidth',3)
datetick('x')
hold on
grid on
xlabel('Time'), ylabel('VaR Value - Dollar'), title('Comparison of Risks')
plot(Date(wwidth:end-1,:),VaRN,'g','LineWidth',3)
plot(Date(wwidth:end-1,:),VaROG,'m','LineWidth',3)
plot(Date(wwidth:end-1,:),VaRI,'b','LineWidth',3)
subplot(2,1,2)
axis([0 21 -0.20 0.20])
plot(Date(wwidth:end-1,:),VaRQ./CAP(2:end),'r','LineWidth',3)
datetick('x')
hold on
grid on
xlabel('Time'), ylabel('VaR Value - Percentage'), title('Comparison of Risks')
plot(Date(wwidth:end-1,:),VaRN./CAPN(2:end),'g','LineWidth',3)
plot(Date(wwidth:end-1,:),VaROG./OGCAPITST(2:end),'m','LineWidth',3)
plot(Date(wwidth:end-1,:),VaRI./INDCAPITST(2:end),'b','LineWidth',3)
hold off